-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c05_s010_extract.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   114498 - 5/15/03 - tdm - Cartesian product join in mySAP TEDW extract sql
-- #   115644 - 6/09/03 - tdm - mySAP.com wep not processing exceptions correctly
-- ############################################################################

-- Job summary
-- 1. retrieve all records from r3_progresp during collection period into temp table
-- 2. move all rejected multicustomer records into exception table
-- Note:
-- perform join on r3_time table to format time into proper format

-- 1. retrieve all records from r3_progresp during collection period into temp table
--#IGNORE_ERROR          
drop table abh.temp_extract_control
;

-- create table to hold the extract window on the source DB
create table abh.temp_extract_control (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_SOURCE
insert into abh.temp_extract_control
select 
extctl_from_intseq,
extctl_from_intseq
from 
twg.extract_control
where
extctl_source='ABH.STAGE_DB' and
extctl_target='ABH.STAGE_PROGRESP'
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update abh.temp_extract_control
set extctl_to_intseq = (select max(prog_seq_id) from r3_progresp)
where (select count(*) from r3_progresp) > 0;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_progresp_cpy;
--#EXECUTE_AT_TARGET
create table abh.stage_progresp_cpy like abh.stage_progresp;
--#INSERT_INTO_TARGET
INSERT into abh.stage_progresp_cpy
SELECT
   r3_progresp.record_type,
   r3_progresp.system_label,
   r3_progresp.sys_id,
   r3_progresp.rfc_dest_str,
   r3_progresp.host_name,
   r3_progresp.diag_date,
   r3_time.long_time,
   r3_progresp.t_code,
   r3_progresp.cua_prog,
   r3_progresp.user_name,
   r3_progresp.task_type,
   r3_progresp.diag_count,
   r3_progresp.cpu_time,
   r3_progresp.response_time,
   r3_progresp.database_time,
   r3_progresp.other_time,
   r3_progresp.kb_transfered,
   r3_progresp.kb_changed,
   r3_progresp.wait_time,
   r3_progresp.memory_used,
   r3_progresp.load_time,
   r3_progresp.gui_time,
   r3_progresp.gui_count,
   r3_progresp.gui_net_time,
   r3_progresp.prog_seq_id
FROM
   r3_progresp LEFT OUTER JOIN r3_time
      ON ( r3_progresp.diag_hour = r3_time.short_time ),
   abh.temp_extract_control ec
WHERE
  r3_progresp.prog_seq_id > ec.extctl_from_intseq and
  r3_progresp.prog_seq_id <= ec.extctl_to_intseq
ORDER BY
   r3_progresp.record_type,
   r3_progresp.system_label,
   r3_progresp.sys_id,
   r3_progresp.rfc_dest_str,
   r3_progresp.host_name,
   r3_progresp.diag_date,
   r3_time.long_time,
   r3_progresp.t_code,
   r3_progresp.cua_prog,
   r3_progresp.user_name,
   r3_progresp.task_type;

--drop and re-create to_intseq temp table in CDW
--(we create a temp table because we need to update the EC table in the CDW
-- but we don't have cross DB update select in execsql, only insert select)
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_to_intseq
;
--#EXECUTE_AT_TARGET
create table abh.temp_to_intseq (
  extctl_to_intseq   integer not null) 
;

-- copy to_intseq value from temp EC table in source DB to CDW
--#INSERT_INTO_TARGET
insert into abh.temp_to_intseq 
select extctl_to_intseq from abh.temp_extract_control
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DB' and
  extctl_target='ABH.STAGE_PROGRESP'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DB' and
 ec.extctl_target='ABH.STAGE_PROGRESP' 
;


-- 2. move all rejected multicustomer records into exception table
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_extract_control
;

-- create table to hold the extract window on the source DB
--#EXECUTE_AT_TARGET
create table abh.temp_extract_control (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_TARGET
insert into abh.temp_extract_control
select 
 extctl_from_intseq,
 extctl_to_intseq
from 
 abh.temp_extract_control
;

--#EXECUTE_AT_TARGET
insert into abh.except_progresp 
select
 n.record_type,
 n.system_label,
 n.sys_id,
 n.rfc_dest_str,
 n.ip_host,
 n.diag_date,
 n.diag_hour,
 n.t_code,
 n.cua_prog,
 n.user_name,
 n.task_type,
 n.diag_count,
 n.cpu_time,
 n.response_time,
 n.database_time,
 n.other_time,
 n.kb_transfered,
 n.kb_changed,
 n.wait_time,
 n.memory_used,
 n.load_time,
 n.gui_time,
 n.gui_count,
 n.gui_net_time,
 n.prog_seq_id
from
 abh.stage_progresp_cpy n, 
 abh.centr_lookup centr_l,
 abh.temp_extract_control ec
where 
  not exists(
    select 1 from abh.centr_lookup 
    where abh.centr_lookup.value = '@' OR 
          abh.centr_lookup.value = n.system_label) and
  n.prog_seq_id > ec.extctl_from_intseq and
  n.prog_seq_id <= ec.extctl_to_intseq
;

--#EXECUTE_AT_TARGET
insert into abh.except_progresp 
select
 n.record_type,
 n.system_label,
 n.sys_id,
 n.rfc_dest_str,
 n.ip_host,
 n.diag_date,
 n.diag_hour,
 n.t_code,
 n.cua_prog,
 n.user_name,
 n.task_type,
 n.diag_count,
 n.cpu_time,
 n.response_time,
 n.database_time,
 n.other_time,
 n.kb_transfered,
 n.kb_changed,
 n.wait_time,
 n.memory_used,
 n.load_time,
 n.gui_time,
 n.gui_count,
 n.gui_net_time,
 n.prog_seq_id
from
 abh.stage_progresp_cpy n, 
 abh.cust_lookup cust_l,
 abh.temp_extract_control ec
where 
  not exists(
    select 1 from abh.cust_lookup 
    where abh.cust_lookup.value = '@' OR 
          abh.cust_lookup.value = n.system_label)  and
  n.prog_seq_id > ec.extctl_from_intseq and
  n.prog_seq_id <= ec.extctl_to_intseq
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='ABH.EXCEPT_PROGRESP'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_PROGRESP' and
 ec.extctl_target='ABH.EXCEPT_PROGRESP' 
;
