-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c10_s010_extract.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   114498 - 5/15/03 - tdm - Cartesian product join in mySAP TEDW extract sql
-- #   115644 - 6/09/03 - tdm - mySAP.com wep not processing exceptions correctly
-- ############################################################################

-- Job summary
-- 1. retrieve all records from r3_disk during collection period into temp table
-- 2. move all rejected multicustomer records into exception table
-- Note:
-- perform join on r3_time table to format time into proper format
-- drop and re-create the tmp_ExtractContrl table to flush it

-- 1. retrieve all records from r3_disk during collection period into temp table
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_SOURCE
insert into abh.tmp_ExtractContrl
select 
extctl_from_intseq,
extctl_from_intseq
from 
twg.extract_control
where
extctl_source='ABH.STAGE_DB' and
extctl_target='ABH.STAGE_DISK'
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update abh.tmp_ExtractContrl
set extctl_to_intseq = (select max(disk_seq_id) from r3_disk)
where (select count(*) from r3_disk) > 0;


--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_disk_copy;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop trigger abh.snap_disk_trig;

--#EXECUTE_AT_TARGET
create table abh.stage_disk_copy like abh.stage_disk;

--#EXECUTE_AT_TARGET
CREATE TRIGGER abh.snap_disk_trig
NO CASCADE
BEFORE INSERT ON abh.stage_disk_copy  
REFERENCING NEW AS N
FOR EACH ROW
MODE DB2SQL
SET N.disk_seq_id = NEXTVAL for abh.stage_disk_seq
;

--#INSERT_INTO_TARGET
insert into abh.stage_disk_copy
SELECT
   r3_disk.host_name,
   r3_disk.system_label,
   r3_disk.snap_date,
   r3_time.long_time,
   sum(r3_disk.snap_count),
   r3_disk.disk_name,
   sum(r3_disk.utilization),
   sum(r3_disk.queue_length),
   sum(r3_disk.wait),
   sum(r3_disk.service),
   0
FROM
   r3_disk RIGHT OUTER JOIN r3_time
      ON ( r3_disk.snap_hour = r3_time.short_time ),
   abh.tmp_ExtractContrl ec
WHERE
   r3_disk.disk_seq_id > ec.extctl_from_intseq and
   r3_disk.disk_seq_id <= ec.extctl_to_intseq
GROUP BY
   r3_disk.host_name,
   r3_disk.system_label,
   r3_disk.snap_date,
   r3_time.long_time,
   r3_disk.disk_name
;

--drop and re-create to_intseq temp table in CDW
--(we create a temp table because we need to update the EC table in the CDW
-- but we don't have cross DB update select in execsql, only insert select)
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_to_intseq
;
--#EXECUTE_AT_TARGET
create table abh.temp_to_intseq (
  extctl_to_intseq   integer not null) 
;

-- copy to_intseq value from temp EC table in source DB to CDW
--#INSERT_INTO_TARGET
insert into abh.temp_to_intseq 
select extctl_to_intseq from abh.tmp_ExtractContrl
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DB' and
  extctl_target='ABH.STAGE_DISK'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DB' and
 ec.extctl_target='ABH.STAGE_DISK' 
;

-- 2. move all rejected multicustomer records into exception table
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
--#EXECUTE_AT_TARGET
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_TARGET
insert into abh.tmp_ExtractContrl
select 
 extctl_from_intseq,
 extctl_to_intseq
from 
 abh.tmp_ExtractContrl
;

--#EXECUTE_AT_TARGET
insert into abh.except_disk 
select
 n.ip_host,
 n.system_label,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.disk_name,
 n.utilization,
 n.queue_length,
 n.wait,
 n.service,
 n.disk_seq_id
from
 abh.stage_disk_copy n, 
 abh.centr_lookup centr_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.centr_lookup 
    where abh.centr_lookup.value = '@' OR 
          abh.centr_lookup.value = n.system_label) and
  n.disk_seq_id > ec.extctl_from_intseq and
  n.disk_seq_id <= ec.extctl_to_intseq
;

--#EXECUTE_AT_TARGET
insert into abh.except_disk 
select
 n.ip_host,
 n.system_label,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.disk_name,
 n.utilization,
 n.queue_length,
 n.wait,
 n.service,
 n.disk_seq_id
from
 abh.stage_disk_copy n, 
 abh.cust_lookup cust_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.cust_lookup 
    where abh.cust_lookup.value = '@' OR 
          abh.cust_lookup.value = n.system_label)  and
  n.disk_seq_id > ec.extctl_from_intseq and
  n.disk_seq_id <= ec.extctl_to_intseq
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='ABH.EXCEPT_DISK'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DISK' and
 ec.extctl_target='ABH.EXCEPT_DISK' 
;
