-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c10_s050_load.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   116675 - 6/30/03 - tdm - mySAP wep perfomance enhancement in load steps
-- ############################################################################

-- 1.   Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a. Create/load temp_system_label w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--  1b. Insert data from temp_system_label into TWG.COMP where does not exist
-- 2.   Insert all newly encountered IP_HOST(s) into the CDW
--  2a. Create/load temp_ip_host w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--  2b. Insert data from temp_ip_host into TWG.COMP where does not exist
--  2c. Insert new relationships into TWG.CompReln
-- 3.   Insert all newly encountered ABH_DISK(s) into the CDW
--  3a. Create/load temp_disk w/TWG.Cur_Comp.CompTyp_Cd='ABH_DISK'
--  3b. Insert data from temp_disk into TWG.COMP where does not exist
--  3c. Insert new relationships into TWG.CompReln
-- 4.   Update measurement table for 'Service time'
--  4a. Update 'Service time' measurement
--  4b. Update extract control table
-- 5.   Update measurement table for 'Disk wait time'
--  5a. Update 'Disk wait time' measurement
--  5b. Update extract control table
-- 6.  Update measurement table for 'Disk utilization'
--  6a. Update 'Disk utilization' measurement
--  6b. Update extract control table
-- 7.  Update measurement table for 'Queue length'
--  7a. Update 'Queue length' measurement
--  7b. Update extract control table

-- 1. Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a. Create/load temp_before_cdw_systems w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--#EXECUTE_AT_TARGET
declare global temporary table temp_system_label (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_system_label
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val
from TWG.Cur_Comp
where CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_systems (
        system_label           varchar(34)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_systems
select
 distinct(system_label)
from abh.stage_disk_copy
;

--  1b. Insert data from temp_before_cdw_systems into TWG.COMP where does not exist

--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP (Comp_Id, CompTyp_Cd, Centr_Cd, Cust_ID, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm, Comp_Ds)
select
 0 as Comp_Id,
 'ABH_SYSTEM' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 n.system_label as comp_nm,
 current timestamp - current timezone as Comp_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_systems n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 n.system_label != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and not exists (
  select 1 
  from
   session.temp_system_label c
  where
   c.comp_nm = n.system_label)
;

-- 2.   Insert all newly encountered IP_HOST(s) into the CDW
--  2a. Create/load temp_before_cdw_ip_hosts w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--#EXECUTE_AT_TARGET
declare global temporary table temp_ip_host (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_ip_host
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_sys_ip (
        system_label           varchar(34),
        ip_host                varchar(55)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_sys_ip
select
 system_label,
 ip_host
from abh.stage_disk_copy
group by 
 system_label,
 ip_host
;

--  2b. Insert data from temp_before_cdw_ip_hosts into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'IP_HOST' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.ip_host as comp_nm,
 n.system_label as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_sys_ip n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_ip_host comp  
where 
 n.ip_host != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.system_label
 and not exists (
  select 1 
  from
   session.temp_ip_host c
  where
   c.comp_nm = n.ip_host and
   c.comp_corr_val = n.system_label 
  )
;

--  2c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'IP_HOST'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 3.   Insert all newly encountered ABH_DISKS(s) into the CDW
--  3a. Create/load temp_disk w/TWG.Cur_Comp.CompTyp_Cd='ABH_DISK'
--#EXECUTE_AT_TARGET
declare global temporary table temp_disk (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_disk
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'ABH_DISK'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_disk (
        system_label           varchar(34),
        ip_host                varchar(55),
        disk_name              varchar(34)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_disk
select
 system_label,
 ip_host,
 disk_name
from abh.stage_disk_copy
group by 
 system_label,
 ip_host,
 disk_name
;

--  3b. Insert data from temp_disk into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_DISK' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.disk_name as comp_nm,
 n.ip_host as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_disk n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_disk comp  
where 
 n.disk_name != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.ip_host
 and comp.comp_Corr_Val = n.system_label
 and not exists (
  select 1 
  from
   session.temp_disk c1,
   session.temp_disk c2
  where
   c1.comp_nm = n.disk_name and
   c1.comp_corr_id = c2.comp_id and
   c2.comp_nm = n.ip_host and
   c2.comp_corr_val = n.system_label
  )
;


--  3c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_DISK'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');

-- 4.   Update measurement table
--  4a. Update 'Service time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtService_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtService_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(disk_seq_id) from abh.stage_disk_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_DISK' and
   a.extctl_target = 'TWG.MSMT_SERVICE_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  sum(d.service)/sum(d.snap_count)    as Msmt_Avg_Val, 
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_disk_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtService_sq t_seq
where
  comp1.Comp_Nm = d.disk_name and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Service time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_DISK' and
  d.disk_seq_id > t_seq.SeqID_from and 
  d.disk_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  4b. Update extract log table
--      Note: the exists clause is required for restarts, the stage table may be empty
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtService_sq)
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_SERVICE_TIME' and
  exists (select 1 from session.tmp_MsmtService_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_SERVICE_TIME'
;


-- 5.   Update 'Disk wait time' measurement
--  5a. Update 'Disk wait time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtDskWait_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtDskWait_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(disk_seq_id) from abh.stage_disk_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_DISK' and
   a.extctl_target = 'TWG.MSMT_WAIT_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  sum(d.wait)/sum(d.snap_count)   as Msmt_Avg_Val,
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_disk_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtDskWait_sq t_seq
where
  comp1.Comp_Nm = d.disk_name and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Disk wait time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_DISK' and
  d.disk_seq_id > t_seq.SeqID_from and 
  d.disk_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  5b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtDskWait_sq)
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_WAIT_TIME' and
  exists (select 1 from session.tmp_MsmtDskWait_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_WAIT_TIME'
;


-- 6.  Update 'Disk utilization' measurement
--  6a. Update 'Disk utilization' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_msmt_util_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_msmt_util_seq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(disk_seq_id) from abh.stage_disk_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_DISK' and
   a.extctl_target = 'TWG.MSMT_DISK_UTIL'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  sum(d.utilization)/sum(d.snap_count)   as Msmt_Avg_Val,
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_disk_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_msmt_util_seq t_seq
where
  comp1.Comp_Nm = d.disk_name and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Disk utilization' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_DISK' and
  d.disk_seq_id > t_seq.SeqID_from and 
  d.disk_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  6b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_msmt_util_seq)
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_DISK_UTIL' and
  exists (select 1 from session.temp_msmt_util_seq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_DISK_UTIL'
;


-- 7.  Update 'Queue length' measurement
--  7a. Update 'Queue length' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtQueue_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtQueue_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(disk_seq_id) from abh.stage_disk_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_DISK' and
   a.extctl_target = 'TWG.MSMT_QUEUE_LENGTH'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  sum(d.queue_length)/sum(d.snap_count)   as Msmt_Avg_Val,
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  ABH.stage_disk_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtQueue_sq t_seq
where
  comp1.Comp_Nm = d.disk_name and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Queue length' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_DISK' and
  d.disk_seq_id > t_seq.SeqID_from and 
  d.disk_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  7b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtQueue_sq)
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_QUEUE_LENGTH' and
  exists (select 1 from session.tmp_MsmtQueue_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_DISK' and
  extctl_target='TWG.MSMT_QUEUE_LENGTH'
;

