-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c15_s010_extract.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   114498 - 5/15/03 - tdm - Cartesian product join in mySAP TEDW extract sql
-- #   115644 - 6/09/03 - tdm - mySAP.com wep not processing exceptions correctly
-- ############################################################################

-- Job summary
-- 1. retrieve all records from r3_cpu during collection period into temp table
-- 2. move all rejected multicustomer records into exception table
-- Note:
-- perform join on r3_time table to format time into proper format
-- drop and re-create the temp_extract_control table to flush it

-- 1. retrieve all records from r3_cpu during collection period into temp table
--#IGNORE_ERROR          
drop table abh.temp_extract_control
;

-- create table to hold the extract window on the source DB
create table abh.temp_extract_control (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_SOURCE
insert into abh.temp_extract_control
select 
extctl_from_intseq,
extctl_from_intseq
from 
twg.extract_control
where
extctl_source='ABH.STAGE_DB' and
extctl_target='ABH.STAGE_CPU'
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update abh.temp_extract_control
set extctl_to_intseq = (select max(cpu_seq_id) from r3_cpu)
where (select count(*) from r3_cpu) > 0;


--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_cpu_copy;

--#EXECUTE_AT_TARGET
create table abh.stage_cpu_copy like abh.stage_cpu;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop trigger abh.snap_cpu_trig;

--#EXECUTE_AT_TARGET
CREATE TRIGGER abh.snap_cpu_trig
NO CASCADE
BEFORE INSERT ON abh.stage_cpu_copy  
REFERENCING NEW AS N
FOR EACH ROW
MODE DB2SQL
SET N.cpu_seq_id = NEXTVAL for abh.stage_cpu_seq
;

--#INSERT_INTO_TARGET
insert into abh.stage_cpu_copy
SELECT
   r3_cpu.host_name,
   r3_cpu.system_label,
   r3_cpu.snap_date,
   r3_time.long_time,
   sum(r3_cpu.snap_count),
   sum(r3_cpu.user_cpu),
   sum(r3_cpu.system_cpu),
   sum(r3_cpu.system_cpu) + sum(r3_cpu.user_cpu),
   sum(r3_cpu.idle_cpu),
   sum(r3_cpu.num_users),
   0
FROM
   r3_cpu RIGHT OUTER JOIN r3_time
      ON ( r3_cpu.snap_hour = r3_time.short_time ),
   abh.temp_extract_control ec
WHERE
   r3_cpu.cpu_seq_id > ec.extctl_from_intseq and
   r3_cpu.cpu_seq_id <= ec.extctl_to_intseq 
GROUP BY
   r3_cpu.host_name,
   r3_cpu.system_label,
   r3_cpu.snap_date,
   r3_time.long_time
;

--drop and re-create to_intseq temp table in CDW
--(we create a temp table because we need to update the EC table in the CDW
-- but we don't have cross DB update select in execsql, only insert select)
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_to_intseq
;
--#EXECUTE_AT_TARGET
create table abh.temp_to_intseq (
  extctl_to_intseq   integer not null) 
;

-- copy to_intseq value from temp EC table in source DB to CDW
--#INSERT_INTO_TARGET
insert into abh.temp_to_intseq 
select extctl_to_intseq from abh.temp_extract_control
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DB' and
  extctl_target='ABH.STAGE_CPU'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DB' and
 ec.extctl_target='ABH.STAGE_CPU' 
;

-- 2. move all rejected multicustomer records into exception table
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_extract_control
;

-- create table to hold the extract window on the source DB
--#EXECUTE_AT_TARGET
create table abh.temp_extract_control (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_TARGET
insert into abh.temp_extract_control
select 
 extctl_from_intseq,
 extctl_to_intseq
from 
 abh.temp_extract_control
;

--#EXECUTE_AT_TARGET
insert into abh.except_cpu 
select
 n.ip_host,
 n.system_label,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.user_cpu,
 n.system_cpu,
 n.total_cpu,
 n.idle_CPU,
 n.num_users,
 n.cpu_seq_id
from
 abh.stage_cpu_copy n, 
 abh.centr_lookup centr_l,
 abh.temp_extract_control ec
where 
  not exists(
    select 1 from abh.centr_lookup 
    where abh.centr_lookup.value = '@' OR 
          abh.centr_lookup.value = n.system_label) and
    n.cpu_seq_id > ec.extctl_from_intseq and
    n.cpu_seq_id <= ec.extctl_to_intseq 
;

--#EXECUTE_AT_TARGET
insert into abh.except_cpu 
select
 n.ip_host,
 n.system_label,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.user_cpu,
 n.system_cpu,
 n.total_cpu,
 n.idle_CPU,
 n.num_users,
 n.cpu_seq_id
from
 abh.stage_cpu_copy n, 
 abh.cust_lookup cust_l,
 abh.temp_extract_control ec
where 
  not exists(
    select 1 from abh.cust_lookup 
    where abh.cust_lookup.value = '@' OR 
          abh.cust_lookup.value = n.system_label)  and
    n.cpu_seq_id > ec.extctl_from_intseq and
    n.cpu_seq_id <= ec.extctl_to_intseq 
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_CPU' and
  extctl_target='ABH.EXCEPT_CPU'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_CPU' and
 ec.extctl_target='ABH.EXCEPT_CPU' 
;
