-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c20_s050_load.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   115645 - 6/09/03 - tdm - mySAP wep perfomance enhancement in load steps
-- #   116675 - 6/30/03 - tdm - mySAP wep perfomance enhancement in load steps
-- ############################################################################

-- 1.    Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a.  Create/load temp_system_label w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--  1b.  Insert data from temp_system_label into TWG.COMP where does not exist
-- 2.    Insert all newly encountered GROUP_NAME(s) into the CDW
--  2a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_APPLNAME'
--  2b.  Insert data from temp table into TWG.COMP where does not exist
--  2c.  Insert new relationships into TWG.CompReln
-- 3.    Insert all newly encountered ABH_TRANSACTION(s) into the CDW
--  3a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_TRANSACTION'
--  3b.  Insert data from temp table into TWG.COMP where does not exist
--  3c.  Insert new relationships into TWG.CompReln
-- 4.   Update measurement table for 'Within SLA'
--  4a. Update 'Within SLA' measurement
--  4b. Update extract control table


-- 1. Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a. Create/load temp_before_cdw_systems w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--#EXECUTE_AT_TARGET
declare global temporary table temp_system_label (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_system_label
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val
from TWG.Cur_Comp
where CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_systems (
        system_label           varchar(34)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_systems
select
 distinct(system_label)
from abh.stage_tx_sla_copy
;

--  1b. Insert data from temp_before_cdw_systems into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP (Comp_Id, CompTyp_Cd, Centr_Cd, Cust_ID, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm, Comp_Ds)
select
 0 as Comp_Id,
 'ABH_SYSTEM' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 n.system_label as comp_nm,
 current timestamp - current timezone as Comp_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_systems n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 n.system_label != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and not exists (
  select 1 
  from
   session.temp_system_label c
  where
   c.comp_nm = n.system_label)
;

-- 2.    Insert all newly encountered GROUP_NAME(s) into the CDW
--  2a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_APPLNAME'
--#EXECUTE_AT_TARGET
declare global temporary table temp_group (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_group
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_APPLNAME' OR
 CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table tmp_UniqueSys_Grp (
        system_label           varchar(34),
        group_name             varchar(80)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_UniqueSys_Grp
select
 system_label,
 group_name
from abh.stage_tx_sla_copy 
group by 
 system_label,
 group_name
;

--  2b.  Insert data from temp table into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_APPLNAME' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.group_name as comp_nm,
 n.system_label as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.tmp_UniqueSys_Grp n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_group comp  
where 
 n.group_name != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.system_label
 and not exists (
  select 1 
  from
   session.temp_group c
  where
   c.comp_nm = n.group_name
   and c.comp_corr_val = n.system_label )
;

--  2c.  Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_APPLNAME'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 3.    Insert all newly encountered ABH_TRANSACTION(s) into the CDW
--  3a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_TRANSACTION'
--#EXECUTE_AT_TARGET
declare global temporary table temp_trans (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_trans
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_TRANSACTION' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'ABH_APPLNAME'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_tcode (
        system_label           varchar(34),
        group_name             varchar(80),
        t_code                 varchar(20)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_tcode
select
 system_label,
 group_name,
 t_code
from abh.stage_tx_sla_copy 
group by 
 system_label,
 group_name,
 t_code
;

--  3b. Insert data from temp_server into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_TRANSACTION' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.t_code as comp_nm,
 n.group_name as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_tcode n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_trans comp  
where 
 n.t_code != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.group_name
 and comp.comp_Corr_Val = n.system_label
 and not exists (
  select 1 
  from
   session.temp_trans c1,
   session.temp_trans c2
  where
   c1.comp_nm = n.t_code and 
   c1.comp_corr_id = c2.comp_id and
   c2.comp_nm = n.group_name and
   c2.comp_corr_val = n.system_label
  )
;


--  3c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_TRANSACTION'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');

-- 4.   Update measurement table for 'Within SLA'
--  4a. Update 'Within SLA' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_msmt_sla_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_msmt_sla_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_TX_SLA' and
   a.extctl_target = 'TWG.MSMT_WITHIN_SLA' and
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_TX_SLA'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                      as Msmt_ID,
  comp1.Comp_ID          as Comp_ID,
  mtyp.MsmtTyp_ID        as MsmtTyp_ID,
  'H'                    as TmSum_Cd,
  d.record_date          as Msmt_Strt_Dt,
  d.record_hour          as Msmt_Strt_Tm,
  CASE
    WHEN (sum(d.sla_yes_count) * 100)/sum(d.tot_count) > 100 THEN 100
    ELSE (sum(d.sla_yes_count) * 100)/sum(d.tot_count)
  END as Msmt_Avg_Val,
  sum(d.tot_count)      as Msmt_Smpl_Cnt 
from
  ABH.stage_tx_sla_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_msmt_sla_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.group_name and
  comp2.Comp_Corr_Val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Within SLA' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.sla_seq_id > t_seq.SeqID_from and 
  d.sla_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.record_date,
  d.record_hour,
  d.t_code,
  d.system_label,
  d.group_name
;

--  4b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_msmt_sla_seq)
where
  extctl_source='ABH.STAGE_TX_SLA' and
  extctl_target='TWG.MSMT_WITHIN_SLA'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_TX_SLA' and
  extctl_target='TWG.MSMT_WITHIN_SLA'
;
