-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c25_s010_extract.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   114498 - 5/15/03 - tdm - Cartesian product join in mySAP TEDW extract sql
-- #   115644 - 6/09/03 - tdm - mySAP.com wep not processing exceptions correctly
-- ############################################################################

-- Job summary
-- 1. retrieve all records from r3_avail during collection period into temp table
-- 2. move all rejected multicustomer records into exception table
-- 3. retrieve all records from r3_user_sum during collection period into temp table
-- 4. move all rejected multicustomer records into exception table

-- Note:
-- perform join on r3_time table to format time into proper format

-- drop and re-create the tmp_ExtractContrl table to flush it

-- 1. retrieve all records from r3_avail during collection period into temp table
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_SOURCE
insert into abh.tmp_ExtractContrl
select 
extctl_from_intseq,
extctl_from_intseq
from 
twg.extract_control
where
extctl_source='ABH.STAGE_DB' and
extctl_target='ABH.STAGE_AVAIL'
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update abh.tmp_ExtractContrl
set extctl_to_intseq = (select max(avail_seq_id) from r3_avail_sum)
where (select count(*) from r3_avail_sum) > 0;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_avail_copy;
--#EXECUTE_AT_TARGET
create table abh.stage_avail_copy like abh.stage_avail;
--#INSERT_INTO_TARGET
insert into abh.stage_avail_copy 
select 
   r3_avail_sum.rfc_dest_str,
   r3_avail_sum.host_name,
   r3_avail_sum.system_label,
   r3_avail_sum.sys_id,
   r3_avail_sum.snap_date,
   r3_time.long_time,
   r3_avail_sum.snap_count,
   r3_avail_sum.sys_available,
   r3_avail_sum.avail_seq_id
FROM
   r3_avail_sum RIGHT OUTER JOIN r3_time
      ON ( r3_avail_sum.snap_hour = r3_time.short_time ),
   abh.tmp_ExtractContrl ec
WHERE
   r3_avail_sum.avail_seq_id > ec.extctl_from_intseq and
   r3_avail_sum.avail_seq_id <= ec.extctl_to_intseq 
ORDER BY
   r3_avail_sum.rfc_dest_str,
   r3_avail_sum.host_name,
   r3_avail_sum.system_label,
   r3_avail_sum.sys_id,
   r3_avail_sum.snap_date,
   r3_time.long_time
;

--drop and re-create to_intseq temp table in CDW
--(we create a temp table because we need to update the EC table in the CDW
-- but we don't have cross DB update select in execsql, only insert select)
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_to_intseq
;
--#EXECUTE_AT_TARGET
create table abh.temp_to_intseq (
  extctl_to_intseq   integer not null) 
;

-- copy to_intseq value from temp EC table in source DB to CDW
--#INSERT_INTO_TARGET
insert into abh.temp_to_intseq 
select extctl_to_intseq from abh.tmp_ExtractContrl
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DB' and
  extctl_target='ABH.STAGE_AVAIL'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DB' and
 ec.extctl_target='ABH.STAGE_AVAIL' 
;

-- 2. move all rejected multicustomer records into exception table
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
--#EXECUTE_AT_TARGET
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_TARGET
insert into abh.tmp_ExtractContrl
select 
 extctl_from_intseq,
 extctl_from_intseq
from 
 abh.tmp_ExtractContrl
;

--#EXECUTE_AT_TARGET
insert into abh.except_avail
select
 n.rfc_dest_str,
 n.ip_host,
 n.system_label,
 n.sys_id,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.sys_available,
 n.avail_seq_id
from
 abh.stage_avail n, 
 abh.centr_lookup centr_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.centr_lookup 
    where abh.centr_lookup.value = '@' OR 
          abh.centr_lookup.value = n.system_label) and
    n.avail_seq_id > ec.extctl_from_intseq and
    n.avail_seq_id <= ec.extctl_to_intseq 
;

--#EXECUTE_AT_TARGET
insert into abh.except_avail
select
 n.rfc_dest_str,
 n.ip_host,
 n.system_label,
 n.sys_id,
 n.snap_date,
 n.snap_hour,
 n.snap_count,
 n.sys_available,
 n.avail_seq_id
from
 abh.stage_avail n, 
 abh.cust_lookup cust_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.cust_lookup 
    where abh.cust_lookup.value = '@' OR 
          abh.cust_lookup.value = n.system_label)  and
    n.avail_seq_id > ec.extctl_from_intseq and
    n.avail_seq_id <= ec.extctl_to_intseq 
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_AVAIL' and
  extctl_target='ABH.EXCEPT_AVAIL'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_AVAIL' and
 ec.extctl_target='ABH.EXCEPT_AVAIL' 
;

-- 3. retrieve all records from r3_user_sum during collection period into temp table
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_SOURCE
insert into abh.tmp_ExtractContrl
select 
extctl_from_intseq,
extctl_from_intseq
from 
twg.extract_control
where
extctl_source='ABH.STAGE_DB' and
extctl_target='ABH.STAGE_USER_SUM'
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update abh.tmp_ExtractContrl
set extctl_to_intseq = (select max(user_seq_id) from r3_user_sum)
where (select count(*) from r3_user_sum) > 0;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_user_sum_cpy;
--#EXECUTE_AT_TARGET
create table abh.stage_user_sum_cpy like abh.stage_user_sum;
--#INSERT_INTO_TARGET
insert into abh.stage_user_sum_cpy 
select 
   r3_user_sum.record_type,
   r3_user_sum.system_label,
   r3_user_sum.host_name,
   r3_user_sum.rfc_dest_str,
   r3_user_sum.user_count,
   r3_user_sum.diag_date,
   r3_time.long_time,
   r3_user_sum.user_seq_id
FROM
   r3_user_sum RIGHT OUTER JOIN r3_time
      ON ( r3_user_sum.diag_hour = r3_time.short_time ),
   abh.tmp_ExtractContrl ec
WHERE
   r3_user_sum.user_seq_id > ec.extctl_from_intseq and
   r3_user_sum.user_seq_id <= ec.extctl_to_intseq 
ORDER BY
   r3_user_sum.system_label,
   r3_user_sum.host_name,
   r3_user_sum.rfc_dest_str,
   r3_user_sum.diag_date,
   r3_time.long_time
;

--drop and re-create to_intseq temp table in CDW
--(we create a temp table because we need to update the EC table in the CDW
-- but we don't have cross DB update select in execsql, only insert select)
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.temp_to_intseq
;
--#EXECUTE_AT_TARGET
create table abh.temp_to_intseq (
  extctl_to_intseq   integer not null) 
;

-- copy to_intseq value from temp EC table in source DB to CDW
--#INSERT_INTO_TARGET
insert into abh.temp_to_intseq 
select extctl_to_intseq from abh.tmp_ExtractContrl
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_DB' and
  extctl_target='ABH.STAGE_USER_SUM'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_DB' and
 ec.extctl_target='ABH.STAGE_USER_SUM' 
;

-- 4. move all rejected multicustomer records into exception table
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
drop table abh.tmp_ExtractContrl
;

-- create table to hold the extract window on the source DB
--#EXECUTE_AT_TARGET
create table abh.tmp_ExtractContrl (
  extctl_from_intseq   integer not null,
  extctl_to_intseq   integer not null)
;

-- get the from_intseq from the extract_control in the CDW
-- and copy that to the source DB
--#INSERT_INTO_TARGET
insert into abh.tmp_ExtractContrl
select 
 extctl_from_intseq,
 extctl_to_intseq
from 
 abh.tmp_ExtractContrl
;

--#EXECUTE_AT_TARGET
insert into abh.except_user_sum
select
 n.record_type,
 n.system_label,
 n.ip_host,
 n.rfc_dest_str,
 n.user_count,
 n.diag_date,
 n.diag_hour,
 n.user_seq_id
from
 abh.stage_user_sum_cpy n, 
 abh.centr_lookup centr_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.centr_lookup 
    where abh.centr_lookup.value = '@' OR 
          abh.centr_lookup.value = n.system_label) and
    n.user_seq_id > ec.extctl_from_intseq and
    n.user_seq_id <= ec.extctl_to_intseq 
;

--#EXECUTE_AT_TARGET
insert into abh.except_user_sum
select
 n.record_type,
 n.system_label,
 n.ip_host,
 n.rfc_dest_str,
 n.user_count,
 n.diag_date,
 n.diag_hour,
 n.user_seq_id
from
 abh.stage_user_sum_cpy n, 
 abh.cust_lookup cust_l,
 abh.tmp_ExtractContrl ec
where 
  not exists(
    select 1 from abh.cust_lookup 
    where abh.cust_lookup.value = '@' OR 
          abh.cust_lookup.value = n.system_label)  and
    n.user_seq_id > ec.extctl_from_intseq and
    n.user_seq_id <= ec.extctl_to_intseq
;

-- update CDW extract control with to_intseq value
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select extctl_to_intseq from abh.temp_to_intseq)
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='ABH.EXCEPT_USER_SUM'
;

-- write extract info to extract log
-- this fires the trigger that updates extract control to close the window
--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
 ec.extctl_source='ABH.STAGE_USER_SUM' and
 ec.extctl_target='ABH.EXCEPT_USER_SUM' 
;


