-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c25_s050_load.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   115645 - 6/09/03 - tdm - mySAP wep perfomance enhancement in load steps
-- #   116675 - 6/30/03 - tdm - mySAP wep perfomance enhancement in load steps
-- ############################################################################

-- 1.    Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a.  Create/load temp_system_label w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--  1b.  Insert data from temp_system_label into TWG.COMP where does not exist
-- 2.    Insert all newly encountered IP_HOST(s) into the CDW
--  2a.  Create/load temp_ip_host w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--  2b.  Insert data from temp_ip_host into TWG.COMP where does not exist
--  2c.  Insert new relationships into TWG.CompReln
-- 3.    Insert all newly encountered ABH_SERVERS(s) into the CDW
--  3a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_SERVER'
--  3b.  Insert data from temp table into TWG.COMP where does not exist
--  3c.  Insert new relationships into TWG.CompReln
-- 4.    Update measurement table for 'Logon able'
--  4a.  Update 'Logon able' measurement
--  4b.  Update extract control table
-- 5.    Update measurement table for 'Not logon able'
--  5a.  Update 'Not logon able' measurement
--  5b.  Update extract control table
-- 6.    Update measurement table for 'Sessions'
--  6a.  Update 'Sessions' measurement
--  6b.  Update extract control table
-- 7.    Update measurement table for 'Logins'
--  7a.  Update 'Logins' measurement
--  7b.  Update extract control table
-- 8.    Update measurement table for 'Active users'
--  8a.  Update 'Active users' measurement
--  8b.  Update extract control table


-- 1. Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a. Create/load temp_before_cdw_systems w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--#EXECUTE_AT_TARGET
declare global temporary table temp_system_label (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_system_label
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val
from TWG.Cur_Comp
where CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_systems (
        system_label           varchar(34)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_systems
select
 distinct(system_label)
from abh.stage_avail_copy
;

--  1b. Insert data from temp_before_cdw_systems into TWG.COMP where does not exist

--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP (Comp_Id, CompTyp_Cd, Centr_Cd, Cust_ID, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm, Comp_Ds)
select
 0 as Comp_Id,
 'ABH_SYSTEM' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 n.system_label as comp_nm,
 current timestamp - current timezone as Comp_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_systems n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 n.system_label != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and not exists (
  select 1 
  from
   session.temp_system_label c
  where
   c.comp_nm = n.system_label)
;

-- 2.   Insert all newly encountered IP_HOST(s) into the CDW
--  2a. Create/load temp_before_cdw_ip_hosts w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--#EXECUTE_AT_TARGET
declare global temporary table temp_ip_host (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_ip_host
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_sys_ip (
        system_label           varchar(34),
        ip_host                varchar(55)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_sys_ip
select
 system_label,
 ip_host
from abh.stage_avail_copy 
group by 
 system_label,
 ip_host
;

--  2b. Insert data from temp_before_cdw_ip_hosts into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'IP_HOST' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.ip_host as comp_nm,
 n.system_label as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_sys_ip n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_ip_host comp  
where 
 n.ip_host != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.system_label
 and not exists (
  select 1 
  from
   session.temp_ip_host c
  where
   c.comp_nm = n.ip_host
   and c.comp_corr_val = n.system_label )
;

--  2c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'IP_HOST'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 3.    Insert all newly encountered ABH_SERVER(s) into the CDW
--  3a.  Create/load temp_server w/TWG.Cur_Comp.CompTyp_Cd='ABH_SERVER'
--#EXECUTE_AT_TARGET
declare global temporary table temp_server (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_server
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_SERVER' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'IP_HOST'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_server (
        system_label           varchar(34),
        ip_host                varchar(55),
        rfc_dest_str           varchar(20)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_server
select
 system_label,
 ip_host,
 rfc_dest_str
from abh.stage_avail_copy 
group by 
 system_label,
 ip_host,
 rfc_dest_str
;

--  3b. Insert data from temp_server into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_SERVER' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.rfc_dest_str as comp_nm,
 n.ip_host as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_server n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_server comp  
where 
 n.rfc_dest_str != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.ip_host
 and comp.comp_Corr_Val = n.system_label
 and not exists (
  select 1 
  from
   session.temp_server c1,
   session.temp_server c2
  where
   c1.comp_nm = n.rfc_dest_str and
   c1.comp_corr_id = c2.comp_id and
   c2.comp_nm = n.ip_host and
   c2.comp_corr_val = n.system_label
 )
;


--  3c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_SERVER'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');

-- 4.    Update measurement table for 'Logon able'
--  4a.  Update 'Logon able' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtLogon_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtLogon_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(avail_seq_id) from ABH.stage_avail_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_AVAILABLE' and
   a.extctl_target = 'TWG.MSMT_LOGONABLE'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  CASE
    WHEN (sum(d.sys_available) * 100)/sum(d.snap_count) > 100 THEN 100
    ELSE (sum(d.sys_available) * 100)/sum(d.snap_count)
  END as Msmt_Avg_Val,
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  ABH.stage_avail_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtLogon_sq t_seq
where
  comp1.Comp_Nm = d.rfc_dest_str and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Logon able' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_SERVER' and
  d.avail_seq_id > t_seq.SeqID_from and 
  d.avail_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  4b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtLogon_sq)
where
  extctl_source='ABH.STAGE_AVAILABLE' and
  extctl_target='TWG.MSMT_LOGONABLE' and
  exists (select 1 from session.tmp_MsmtLogon_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_AVAILABLE' and
  extctl_target='TWG.MSMT_LOGONABLE'
;

-- 5.    Update measurement table for 'Not logon able'
--  5a.  Update 'Not logon able' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtNotLog_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtNotLog_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(avail_seq_id) from ABH.stage_avail_copy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_AVAILABLE' and
   a.extctl_target = 'TWG.MSMT_NOT_LOGONABLE'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.snap_date          as Msmt_Strt_Dt,
  d.snap_hour          as Msmt_Strt_Tm,
  ((sum(d.snap_count) - sum(d.sys_available)) * 100)/sum(d.snap_count)    as Msmt_Avg_Val,
  sum(d.snap_count)    as Msmt_Smpl_Cnt 
from
  ABH.stage_avail_copy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtNotLog_sq t_seq
where
  comp1.Comp_Nm = d.rfc_dest_str and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Not logon able' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_SERVER' and
  d.avail_seq_id > t_seq.SeqID_from and 
  d.avail_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.snap_date,
  d.snap_hour
;

--  5b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtNotLog_sq)
where
  extctl_source='ABH.STAGE_AVAILABLE' and
  extctl_target='TWG.MSMT_NOT_LOGONABLE' and
  exists (select 1 from session.tmp_MsmtNotLog_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_AVAILABLE' and
  extctl_target='TWG.MSMT_NOT_LOGONABLE'
;

-- 6.    Update measurement table for 'Sessions'
--  6a.  Update 'Sessions' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtSession_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtSession_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(user_seq_id) from ABH.stage_user_sum_cpy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_USER_SUM' and
   a.extctl_target = 'TWG.MSMT_SESSIONS'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Tot_Val )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.user_count)    as Msmt_Tot_Val
from
  ABH.stage_user_sum_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtSession_sq t_seq
where
  comp1.Comp_Nm = d.rfc_dest_str and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Sessions' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_SERVER' and
  d.user_seq_id > t_seq.SeqID_from and 
  d.user_seq_id <= t_seq.SeqID_to and
  d.record_type = 'S'
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  6b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtSession_sq)
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_SESSIONS' and
  exists (select 1 from session.tmp_MsmtSession_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_SESSIONS'
;

-- 7.    Update measurement table for 'Logins'
--  7a.  Update 'Logins' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtLogins_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtLogins_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(user_seq_id) from ABH.stage_user_sum_cpy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_USER_SUM' and
   a.extctl_target = 'TWG.MSMT_LOGINS'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Tot_Val )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.user_count)    as Msmt_Tot_Val
from
  ABH.stage_user_sum_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtLogins_sq t_seq
where
  comp1.Comp_Nm = d.rfc_dest_str and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Logins' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_SERVER' and
  d.user_seq_id > t_seq.SeqID_from and 
  d.user_seq_id <= t_seq.SeqID_to and
  d.record_type = 'L'
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  7b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtLogins_sq)
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_LOGINS' and
  exists (select 1 from session.tmp_MsmtLogins_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_LOGINS'
;

-- 8.    Update measurement table for 'Active users'
--  8a.  Update 'Active users' measurement
--#EXECUTE_AT_TARGET
declare global temporary table tmp_MsmtActive_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_MsmtActive_sq
select 
 a.extctl_from_intseq as SeqID_from,
 ( select max(user_seq_id) from ABH.stage_user_sum_cpy ) as SeqID_to
from TWG.Extract_Control a
where 
   a.extctl_source = 'ABH.STAGE_USER_SUM' and
   a.extctl_target = 'TWG.MSMT_ACTIVE_USERS'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Tot_Val )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.user_count)    as Msmt_Tot_Val
from
  ABH.stage_user_sum_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.tmp_MsmtActive_sq t_seq
where
  comp1.Comp_Nm = d.rfc_dest_str and
  comp1.Comp_Corr_Id = comp2.Comp_Id and
  comp2.Comp_Nm = d.ip_host and
  comp2.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Active users' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_SERVER' and
  d.user_seq_id > t_seq.SeqID_from and 
  d.user_seq_id <= t_seq.SeqID_to and
  d.record_type = 'A'
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  8b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.tmp_MsmtActive_sq)
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_ACTIVE_USERS' and
  exists (select 1 from session.tmp_MsmtActive_sq where SeqID_to > 0)
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_USER_SUM' and
  extctl_target='TWG.MSMT_ACTIVE_USERS'
;

