-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_c40_s050_load.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- #
-- #   Change history:
-- #   115645 - 6/09/03 - tdm - mySAP wep perfomance enhancement in load steps
-- #   116675 - 6/30/03 - tdm - mySAP wep perfomance enhancement in load steps
-- ############################################################################

-- 1.    Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a.  Create/load temp_system_label w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--  1b.  Insert data from temp_system_label into TWG.COMP where does not exist
-- 2.    Insert all newly encountered IP_HOST(s) into the CDW
--  2a.  Create/load temp_ip_host w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--  2b.  Insert data from temp_ip_host into TWG.COMP where does not exist
--  2c.  Insert new relationships into TWG.CompReln
-- 3.    Insert all newly encountered ABH_SERVERS(s) into the CDW
--  3a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_SERVER'
--  3b.  Insert data from temp table into TWG.COMP where does not exist
--  3c.  Insert new relationships into TWG.CompReln
-- 4.    Insert all newly encountered ABH_USER(s) into the CDW
--  4a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_USER'
--  4b.  Insert data from temp table into TWG.COMP where does not exist
--  4c.  Insert new relationships into TWG.CompReln
-- 5.    Insert all newly encountered ABH_TASKTYPE(s) into the CDW
--  5a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_TASKTYPE'
--  5b.  Insert data from temp table into TWG.COMP where does not exist
--  5c.  Insert new relationships into TWG.CompReln
-- 6.    Insert all newly encountered ABH_TRANSACTION(s) into the CDW
--  6a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_TRANSACTION'
--  6b.  Insert data from temp table into TWG.COMP where does not exist
--  6c.  Insert new relationships into TWG.CompReln
-- 7.    Update measurement table for 'CPU time'
--  7a.  Update 'CPU time' measurement
--  7b.  Update extract control table
-- 8.    Update measurement table for 'Response time'
--  8a.  Update 'Response time' measurement
--  8b.  Update extract control table
-- 9.    Update measurement table for 'Databse time'
--  9a.  Update 'Database time' measurement
--  9b.  Update extract control table
-- 10.   Update measurement table for 'kB transferred'
--  10a. Update 'kB transferred' measurement
--  10b. Update extract control table
-- 11.   Update measurement table for 'kB changed'
--  11a. Update 'kB changed' measurement
--  11b. Update extract control table
-- 12.   Update measurement table for 'Wait time'
--  12a. Update 'Wait time' measurement
--  12b. Update extract control table
-- 13.   Update measurement table for 'Memory used'
--  13a. Update 'Memory used' measurement
--  13b. Update extract control table
-- 14.   Update measurement table for 'Load time'
--  14a. Update 'Load time' measurement
--  14b. Update extract control table
-- 15.   Update measurement table for 'Other time'
--  15a. Update 'Other time' measurement
--  15b. Update extract control table
-- 16.   Update measurement table for 'GUI time'
--  16a. Update 'GUI time' measurement
--  16b. Update extract control table
-- 17.   Update measurement table for 'GUI network time'
--  17a. Update 'GUI network time' measurement
--  17b. Update extract control table
-- 18.   Update measurement table for 'Dialog count'
--  18a. Update 'Dialog count' measurement
--  18b. Update extract control table



-- 1. Insert all newly encountered ABH_SYSTEM(s) into the CDW
--  1a. Create/load temp_before_cdw_systems w/TWG.Cur_Comp.CompTyp_Cd='ABH_SYSTEM'
--#EXECUTE_AT_TARGET
declare global temporary table temp_system_label (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_system_label
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val
from TWG.Cur_Comp
where CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_systems (
        system_label           varchar(34)
   )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_systems
select
 distinct(system_label)
from abh.stage_progresp_cpy
;

--  1b. Insert data from temp_before_cdw_systems into TWG.COMP where does not exist

--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP (Comp_Id, CompTyp_Cd, Centr_Cd, Cust_ID, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm, Comp_Ds)
select
 0 as Comp_Id,
 'ABH_SYSTEM' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 n.system_label as comp_nm,
 current timestamp - current timezone as Comp_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_systems n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 n.system_label != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and not exists (
  select 1 
  from
   session.temp_system_label c
  where
   c.comp_nm = n.system_label)
;

-- 2.   Insert all newly encountered IP_HOST(s) into the CDW
--  2a. Create/load temp_before_cdw_ip_hosts w/TWG.Cur_Comp.CompTyp_Cd='IP_HOST'
--#EXECUTE_AT_TARGET
declare global temporary table temp_ip_host (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_ip_host
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SYSTEM'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_sys_ip (
        system_label           varchar(34),
        ip_host                varchar(55)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_sys_ip
select
 system_label,
 ip_host
from abh.stage_progresp_cpy 
group by 
 system_label,
 ip_host
;

--  2b. Insert data from temp_before_cdw_ip_hosts into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'IP_HOST' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.ip_host as comp_nm,
 n.system_label as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_sys_ip n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_ip_host comp  
where 
 n.ip_host != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.system_label
 and not exists (
  select 1 
  from
   session.temp_ip_host c
  where
   c.comp_nm = n.ip_host
   and c.comp_corr_val = n.system_label )
;

--  2c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'IP_HOST'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 3.    Insert all newly encountered ABH_SERVER(s) into the CDW
--  3a.  Create/load temp_server w/TWG.Cur_Comp.CompTyp_Cd='ABH_SERVER'
--#EXECUTE_AT_TARGET
declare global temporary table temp_server (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_server
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_SERVER' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'IP_HOST'
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_server (
        system_label           varchar(34),
        ip_host                varchar(55),
        rfc_dest_str           varchar(20)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_server
select
 system_label,
 ip_host,
 rfc_dest_str
from abh.stage_progresp_cpy 
group by 
 system_label,
 ip_host,
 rfc_dest_str
;

--  3b. Insert data from temp_server into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_SERVER' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.rfc_dest_str as comp_nm,
 n.ip_host as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_server n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_server comp  
where 
 n.rfc_dest_str != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.ip_host
 and comp.comp_Corr_Val = n.system_label
 and not exists (
  select 1 
  from
   session.temp_server c,
   session.temp_server c1
  where
   c.comp_nm = n.rfc_dest_str
   and c.comp_corr_id = c1.comp_id
   and c1.comp_nm = n.ip_host )
;


--  3c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_SERVER'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');

-- 4.    Insert all newly encountered ABH_USER(s) into the CDW
--  4a.  Create/load temp_user w/TWG.Cur_Comp.CompTyp_Cd='ABH_USER'
--#EXECUTE_AT_TARGET
declare global temporary table temp_user (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_user
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_USER' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SERVER' 
;

--#EXECUTE_AT_TARGET
declare global temporary table temp_unique_user (
        system_label           varchar(34),
        ip_host                varchar(55),
        rfc_dest_str           varchar(20),
        user_name              varchar(30)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_unique_user
select
 system_label,
 ip_host,
 rfc_dest_str,
 user_name
from abh.stage_progresp_cpy 
group by 
 system_label,
 ip_host,
 rfc_dest_str,
 user_name
;

--  4b.  Insert data from temp_user into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_USER' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.user_name as comp_nm,
 n.rfc_dest_str as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.temp_unique_user n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_user comp  
where 
 n.user_name != ''
 and ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.rfc_dest_str
 and comp.comp_Corr_Val = n.ip_host
 and not exists (
  select 1 
  from
   session.temp_user c1,
   session.temp_user c2,
   session.temp_user c3,
   session.temp_user c4
  where
   c1.comp_nm = n.user_name and
   c1.comp_corr_id = c2.comp_id and
   c2.comp_nm = n.rfc_dest_str and
   c2.comp_corr_id = c3.comp_id and
   c3.comp_nm = n.ip_host and
   c3.comp_corr_id = c4.comp_id and
   c4.comp_nm = n.system_label
)
;

--  4c.  Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_USER'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');

-- 5.    Insert all newly encountered ABH_TASKTYPE(s) into the CDW
--  5a.  Create/load temp table w/TWG.Cur_Comp.CompTyp_Cd='ABH_TASKTYPE'
--#EXECUTE_AT_TARGET
declare global temporary table temp_program (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_program
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_TASKTYPE' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SERVER' OR
 CompTyp_Cd = 'ABH_USER' 
;

--#EXECUTE_AT_TARGET
declare global temporary table tmp_UniqueProgram (
        system_label           varchar(34),
        ip_host                varchar(55),
        rfc_dest_str           varchar(20),
        user_name              varchar(30),
        task_type              varchar(1)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_UniqueProgram
select
 system_label,
 ip_host,
 rfc_dest_str,
 user_name,
 task_type
from abh.stage_progresp_cpy 
where 
 task_type != 'X'
group by 
 system_label,
 ip_host,
 rfc_dest_str,
 user_name,
 task_type
;


--  5b.  Insert data from temp table into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_TASKTYPE' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp.Comp_Id as Comp_Corr_Id,
 n.task_type as comp_nm,
 n.user_name as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.tmp_UniqueProgram n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_program comp  
where 
 ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and comp.comp_Nm = n.user_name
 and comp.comp_Corr_Val = n.rfc_dest_str
 and not exists (
  select 1 
  from
   session.temp_program c1,
   session.temp_program c2,
   session.temp_program c3,
   session.temp_program c4
  where
   c1.comp_nm = n.task_type and
   c1.comp_corr_id = c2.comp_id and
   c2.comp_nm = n.user_name and
   c2.comp_corr_id = c3.comp_id and
   c3.comp_nm = n.rfc_dest_str and
   c3.comp_corr_id = c4.comp_id and
   c4.comp_nm = n.ip_host and
   c4.comp_corr_val = n.system_label
   )
;

--  5c.  Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_TASKTYPE'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 6.    Insert all newly encountered ABH_TRANSACTION(s) into the CDW
--  6a.  Create/load temp_program w/TWG.Cur_Comp.CompTyp_Cd='ABH_TRANSACTION'
--#EXECUTE_AT_TARGET
declare global temporary table temp_program (
        Comp_Nm                varchar(254),
        Comp_Corr_Val          varchar(254),
        Comp_Id                integer,
        Comp_Corr_Id           integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_program
select
 Comp_Nm as Comp_Nm,
 Comp_Corr_Val as Comp_Corr_Val,
 Comp_Id as Comp_Id,
 Comp_Corr_Id as Comp_Corr_Id
from TWG.Cur_Comp 
where CompTyp_Cd = 'ABH_TRANSACTION' OR
 CompTyp_Cd = 'ABH_SYSTEM' OR
 CompTyp_Cd = 'IP_HOST' OR
 CompTyp_Cd = 'ABH_SERVER' OR
 CompTyp_Cd = 'ABH_USER' OR
 CompTyp_Cd = 'ABH_TASKTYPE'
;

--#EXECUTE_AT_TARGET
declare global temporary table tmp_UniqueProgram (
        system_label           varchar(34),
        ip_host                varchar(55),
        rfc_dest_str           varchar(20),
        user_name              varchar(30),
        task_type              varchar(1),
        t_code                 varchar(20)
  )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.tmp_UniqueProgram
select
 system_label,
 ip_host,
 rfc_dest_str,
 user_name,
 task_type,
 t_code
from abh.stage_progresp_cpy 
group by 
 system_label,
 ip_host,
 rfc_dest_str,
 user_name,
 task_type,
 t_code
;

--  6b.  Insert data from temp_program into TWG.COMP where does not exist
--#EXECUTE_AT_TARGET
INSERT INTO TWG.COMP
select
 0 as Comp_Id,
 'ABH_TRANSACTION' as CompTyp_Cd,
 centr_l.Centr_Cd as Centr_Cd,
 cust.Cust_ID as Cust_ID,
 comp1.Comp_Id as Comp_Corr_Id,
 n.t_code as comp_nm,
 n.task_type as Comp_Corr_Val,
 current timestamp - current timezone as Comp_Strt_DtTm ,
 '9999-01-01-00.00.00.000000' as Comp_End_DtTm,
 '' as Comp_Ds
from
 session.tmp_UniqueProgram n, 
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.temp_program comp1,  
 session.temp_program comp2,  
 session.temp_program comp3,  
 session.temp_program comp4,  
 session.temp_program comp5  
where 
 ((cust_l.value = n.system_label AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = n.system_label OR centr_l.value = '@')
 and length(rtrim(n.t_code)) > 0
 and comp1.comp_Nm = n.task_type
 and comp1.comp_Corr_Id = comp2.comp_Id
 and comp2.comp_Nm  = n.user_name
 and comp2.comp_Corr_Id = comp3.comp_Id
 and comp3.comp_Nm  = n.rfc_dest_str
 and comp3.comp_Corr_Id = comp4.comp_Id
 and comp4.comp_Nm  = n.ip_host
 and comp4.comp_Corr_Id = comp5.comp_Id
 and comp5.comp_Nm  = n.system_label
;

--  6c. Insert new relationships into TWG.CompReln
--#EXECUTE_AT_TARGET
insert into TWG.CompReln
select 0 as CompReln_Id,
 c.Comp_Corr_Id as Comp_Source_Id,
 c.Comp_Id as Comp_Target_Id,
 'PCHILD' as RelnTyp_Cd,
 current timestamp - current timezone as CompReln_Strt_DtTm,
 '9999-01-01-00.00.00.000000' as CompReln_End_DtTm
from TWG.Cur_Comp c
where
 CompTyp_Cd = 'ABH_TRANSACTION'
 and not exists
  (select 1 from TWG.Cur_CompReln r
   where r.Comp_Source_Id = c.Comp_Corr_Id
     and r.Comp_Target_Id = c.Comp_Id
     and r.RelnTyp_Cd = 'PCHILD');


-- 7.    Update measurement table for 'CPU time'
--  7a.  Update 'CPU time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_cpu_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_cpu_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_CPU_TIME' and
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.cpu_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_cpu_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'CPU time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  7b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_cpu_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_CPU_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_CPU_TIME'
;

-- 8.    Update measurement table for 'Response time'
--  8a.  Update 'Response time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_resp_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_resp_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_RESP_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.response_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_resp_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Response time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  8b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_resp_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_RESP_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_RESP_TIME'
;

-- 9.    Update measurement table for 'Database time'
--  9a.  Update 'Database time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_db_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_db_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_DB_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.database_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_db_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Database time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  9b.  Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_db_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_DB_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_DB_TIME'
;

-- 10.   Update measurement table for 'kB transferred'
--  10a. Update 'kB transferred' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_kb_tran_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_kb_tran_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_KB_TRANS' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.kb_transfered)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_kb_tran_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'kB transferred' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  10b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_kb_tran_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_KB_TRANS'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_KB_TRANS'
;

-- 11.   Update measurement table for 'kB changed'
--  11a. Update 'kB changed' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_kb_change_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_kb_change_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_KB_CHANGED' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.kb_changed)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_kb_change_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'kB changed' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  11b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_kb_change_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_KB_CHANGED'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_KB_CHANGED'
;

-- 12.   Update measurement table for 'Wait time'
--  12a. Update 'Wait time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_wait_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_wait_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_WAIT_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.wait_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_wait_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Wait time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  12b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_wait_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_WAIT_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_WAIT_TIME'
;

-- 13.   Update measurement table for 'Memory used'
--  13a. Update 'Memory used' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_mem_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_mem_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_MEMORY_USED' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.memory_used)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_mem_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Memory used' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  13b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_mem_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_MEMORY_USED'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_MEMORY_USED'
;

-- 14.   Update measurement table for 'Load time'
--  14a. Update 'Load time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_load_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_load_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_LOAD_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.load_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_load_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Load time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  14b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_load_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_LOAD_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_LOAD_TIME'
;

-- 15.   Update measurement table for 'Other time'
--  15a. Update 'Other time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_other_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_other_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_OTHER_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.other_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_other_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Other time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  15b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_other_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_OTHER_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_OTHER_TIME'
;

-- 16.   Update measurement table for 'GUI time'
--  16a. Update 'GUI time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_gui_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_gui_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_GUI_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.gui_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_gui_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'GUI time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  16b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_gui_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_GUI_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_GUI_TIME'
;

-- 17.   Update measurement table for 'GUI network time'
--  17a. Update 'GUI network time' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_gui_net_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_gui_net_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_GUI_NET_TIME' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Avg_Val, Msmt_Smpl_Cnt )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.gui_net_time)/sum(d.diag_count)    as Msmt_Avg_Val,
  sum(d.diag_count)    as Msmt_Smpl_Cnt 
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_gui_net_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'GUI network time' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  17b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_gui_net_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_GUI_NET_TIME'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_GUI_NET_TIME'
;

-- 18.   Update measurement table for 'Dialog count'
--  18a. Update 'Dialog count' measurement
--#EXECUTE_AT_TARGET
declare global temporary table temp_diag_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.temp_diag_seq
select 
 a.extctl_from_intseq as SeqID_from,
 b.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a,
     TWG.Extract_Control b
where 
   a.extctl_source = 'ABH.STAGE_PROGRESP' and
   a.extctl_target = 'TWG.MSMT_TRANS_DIALOG_COUNT' and 
   b.extctl_source = 'ABH.STAGE_DB' and
   b.extctl_target = 'ABH.STAGE_PROGRESP'
;

--#EXECUTE_AT_TARGET
insert into TWG.Msmt ( Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt,
                       Msmt_Strt_Tm, Msmt_Tot_Val )
select
  0                    as Msmt_ID,
  comp1.Comp_ID        as Comp_ID,
  mtyp.MsmtTyp_ID      as MsmtTyp_ID,
  'H'                  as TmSum_Cd,
  d.diag_date          as Msmt_Strt_Dt,
  d.diag_hour          as Msmt_Strt_Tm,
  sum(d.diag_count)    as Msmt_Tot_Val
from
  abh.stage_progresp_cpy d,
  TWG.cur_comp comp1,
  TWG.cur_comp comp2,
  TWG.cur_comp comp3,
  TWG.cur_comp comp4,
  TWG.cur_comp comp5,
  TWG.MsmtTyp mtyp,
  TWG.MsmtRul mrul,
  session.temp_diag_seq t_seq
where
  comp1.Comp_Nm = d.t_code and
  comp1.comp_corr_val = d.task_type and
  comp1.comp_corr_id = comp2.comp_id and
  comp2.comp_corr_val = d.user_name and
  comp2.comp_corr_id = comp3.comp_id and
  comp3.comp_corr_val = d.rfc_dest_str and
  comp3.comp_corr_id = comp4.comp_id and
  comp4.comp_corr_val = d.ip_host and
  comp4.comp_corr_id = comp5.comp_id and
  comp5.comp_corr_val = d.system_label and
  mtyp.MsmtTyp_Nm = 'Dialog count' and 
  mtyp.MsmtTyp_ID = mrul.MsmtTyp_ID and
  mrul.CompTyp_Cd = 'ABH_TRANSACTION' and
  d.prog_seq_id > t_seq.SeqID_from and 
  d.prog_seq_id <= t_seq.SeqID_to
group by
  comp1.Comp_ID,
  mtyp.MsmtTyp_ID,
  d.diag_date,
  d.diag_hour
;

--  18b. Update extract control table
--#EXECUTE_AT_TARGET
update TWG.extract_control
  set extctl_to_intseq = (select SeqID_to from session.temp_diag_seq)
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_DIALOG_COUNT'
;

--#EXECUTE_AT_TARGET
insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='ABH.STAGE_PROGRESP' and
  extctl_target='TWG.MSMT_TRANS_DIALOG_COUNT'
;
