-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_m15_s010_buildMart.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- ############################################################################

-- Clean up, delete old data
-- 1.   insert into the metric dimension table
-- 1a.  update extract control for the ip host metric dimension table
-- 2.   insert into the abh.stage_d_ip_host table
-- 2a.  insert from abh.stage_d_ip_host table into abh.d_ip_host
-- 3.   set up temp tables used to populate abh.stage_f_host_hour_copy
-- 3a.  insert into the abh.stage_f_host_hour_copy table for all measure

--Clean up
--#EXECUTE_AT_TARGET
-- log the pruning of the hourly fact table abh.f_ip_host_hour
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_ip_host_hour',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_ip_host_hour'),
  (select count(*) from abh.f_ip_host_hour f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_hour)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_ip_host_hour')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_ip_host_hour f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_hour)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_ip_host_hour')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the daily fact table abh.f_ip_host_day
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_ip_host_day',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_ip_host_day'),
  (select count(*) from abh.f_ip_host_day f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_ip_host_day')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_ip_host_day f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_ip_host_day')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the weekly fact table abh.f_ip_host_week
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_ip_host_week',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_ip_host_week'),
  (select count(*) from abh.f_ip_host_week f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_ip_host_week')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_ip_host_week f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_ip_host_week')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the monthly fact table abh.f_ip_host_month
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_ip_host_month',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_ip_host_month'),
  (select count(*) from abh.f_ip_host_month f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_ip_host_month')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_ip_host_month f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_ip_host_month')
;

-- 1.    insert into the metric dimension table
-- NOTE: This process will only happen once. We initially insert the data into the
--       staging table in the cdw/dml script.
--       set the high value for the data to be extracted

-- 1a.  update extract control for the ip host metric dimension table
-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

update TWG.Extract_Control set 
 extctl_to_intseq = (select max(metric_id) from ABH.STAGE_D_METRIC),
 extctl_to_dttm = (select max(insert_dttm) from ABH.STAGE_D_METRIC)
where 
 extctl_source='ABH.STAGE_D_METRIC' and
 extctl_target='ABH.D_IP_HOST_METRIC' and
 (select count(*) from ABH.STAGE_D_METRIC) > 0
;

--#INSERT_INTO_TARGET
insert into abh.d_ip_host_metric
select distinct
  d.metric_id as metric_id,
  d.Met_Name as Met_Name,
  d.Met_Desc as Met_Desc,
  d.Met_units as Met_units,
  d.Met_Category as Met_Category,
  d.min_exists as MIN_Exists,
  d.max_exists as MAX_Exists,
  d.avg_exists as AVG_Exists,
  d.total_exists as TOTAL_Exists,
  d.Msrc_Nm as MSrc_Nm
from
  abh.stage_d_metric d,
  twg.extract_control ec
where
  d.met_name in ('System CPU','User CPU','Idle CPU','Users','Total CPU') and
  ec.extctl_source = 'ABH.STAGE_D_METRIC' and
  ec.extctl_target = 'ABH.D_IP_HOST_METRIC' and
  d.metric_id > ec.extctl_from_intseq and
  d.metric_id <= ec.extctl_to_intseq and
  d.met_category not like 'mySAP%'
;

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'ABH.STAGE_D_METRIC' and
 ec.extctl_target = 'ABH.D_IP_HOST_METRIC'
; 

-- 2.  insert into the abh.stage_d_ip_host table
--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_d_ip_host;

--#EXECUTE_AT_TARGET
create table abh.stage_d_ip_host like abh.d_ip_host;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update TWG.extract_control
  set extctl_to_intseq = (select max(comp_id) from twg.cur_comp)
where
  extctl_source='TWG.COMP' and
  extctl_target='ABH.STAGE_IP_HOST'
;

-- Note: The static component ABH_OS_DATA will not be displayed in the RPI. So, the comp_id from ABH_OS_DATA 
--       will be used to identify component IP_HOST
--#INSERT_INTO_TARGET
insert into abh.stage_d_ip_host
select 
 c2.comp_corr_val as system_label,
 c2.comp_nm as ip_host,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as xq_strt_dttm,
 '9999-01-01-00.00.00.000000' as xq_end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 twg.cur_comp c2,
 twg.cur_comp c3,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.extract_control ec,
 twg.cust cust
where 
 ec.extctl_source='TWG.COMP' and
 ec.extctl_target='ABH.STAGE_IP_HOST' and
 c1.comp_id > ec.extctl_from_intseq and 
 c1.comp_id <= ec.extctl_to_intseq and
 c1.comptyp_cd='ABH_OS_DATA' and 
 c1.comp_corr_id=c2.comp_id and
 c2.comptyp_cd='IP_HOST' and 
 c2.comp_corr_id=c3.comp_id and
 c3.comp_nm is not null and 
 c3.comptyp_cd='ABH_SYSTEM'  and 
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='TWG.COMP' and
  extctl_target='ABH.STAGE_IP_HOST'
;

-- 2a. insert from abh.stage_d_ip_host table into abh.d_ip_host
--#EXECUTE_AT_TARGET
insert into abh.d_ip_host
select 
 stage.system_label as system_label,
 stage.ip_host as ip_host,
 stage.cust_acct_cd as cust_acct_cd,
 stage.centr_cd as centr_cd,
 stage.xq_strt_dttm as xq_strt_dttm,
 stage.xq_end_dttm as xq_end_dttm,
 stage.comp_id as comp_id
from 
 abh.stage_d_ip_host stage
where
 not exists (
  select 1 
  from
   abh.d_ip_host c
  where
   c.comp_id = stage.comp_id)
;

-- 3.   set up temp tables used to populate abh.stage_f_host_hour_copy
--      Note: There are (4) measures associated w/ip_host
--      they are 'System CPU','User CPU','Idle CPU','Total CPU' & 'Users'

declare global temporary table stage_ms_ip_host (
 system_label         varchar(34),
 ip_host              varchar(55),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_ms_ip_host
select 
 comp1.comp_corr_val as system_label,
 comp1.comp_nm as ip_host,
 comp.comp_id as comp_id
from
 twg.cur_comp comp,
 twg.cur_comp comp1,
 twg.cur_comp comp2
where
 comp.comptyp_cd = 'ABH_OS_DATA' and
 comp.comp_corr_id = comp1.comp_id and
 comp1.comptyp_cd = 'IP_HOST' and
 comp1.comp_corr_id = comp2.comp_id and
 comp2.comptyp_cd = 'ABH_SYSTEM'
;

-- 3a. insert into the abh.stage_f_host_hour table for measure systemcpu
-- set the high value for the data to be extracted
update TWG.extract_control set 
 extctl_to_intseq = ( select max(msmt_id) from twg.msmt)
where 
 extctl_source='TWG.MSMT' and
 extctl_target='ABH.F_IP_HOST_HOUR' and
 (select count(*) from twg.msmt) > 0
;

declare global temporary table temp_f_ip_host_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

insert into session.temp_f_ip_host_seq
select 
 a.extctl_from_intseq as SeqID_from,
 a.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a
where 
  a.extctl_source='TWG.MSMT' and
  a.extctl_target='ABH.F_IP_HOST_HOUR'
;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_f_host_hour;

--#EXECUTE_AT_TARGET
create table abh.stage_f_host_hour like abh.f_ip_host_hour;

--#INSERT_INTO_TARGET
insert into abh.stage_f_host_hour
select
  ms.comp_id as host_id,
  ms.msmttyp_id as metric_id,
  timestamp(ms.msmt_strt_dt, ms.msmt_strt_tm) as meas_hour,   
  ms.msmt_smpl_cnt as sample_count,
  ms.msmt_min_val as min_value,
  ms.msmt_max_val as max_value,
  ms.msmt_avg_val as avg_value,
  ms.msmt_tot_val as total_value
from
  twg.msmt ms, 
  twg.msmttyp mt, 
  session.stage_ms_ip_host stage,
  session.temp_f_ip_host_seq t_seq
where
  stage.comp_id = ms.comp_id and
  ms.msmt_id > t_seq.SeqID_from and 
  ms.msmt_id <= t_seq.SeqID_to and
  (ms.msmttyp_id = mt.msmttyp_id and mt.msrc_cd = 'ABH' and 
   mt.msmttyp_nm in ('System CPU','User CPU','Idle CPU','Total CPU','Users') ) and
  ms.tmsum_cd = 'H'
;

--#EXECUTE_AT_TARGET
update abh.stage_f_host_hour 
  set total_value = avg_value * sample_count 
where 
  total_value is null;

--#EXECUTE_AT_TARGET
insert into abh.f_ip_host_hour
select * from abh.stage_f_host_hour
;

-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'TWG.MSMT' and
 ec.extctl_target = 'ABH.F_IP_HOST_HOUR'
; 

