-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5724-B97
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: abh_m20_s010_buildMart.db2
-- #   Revision: 1.0.1
-- #   Date: 08/31/2002
-- ############################################################################

-- Clean up, delete old data
-- 1.   insert into the metric dimension table
-- 1a.  update extract control for the transaction metric dimension table
-- 1b.  update extract control for the appl group metric dimension table
-- 1c.  update extract control for the tx sla metric dimension table
-- 2.   insert into the abh.stage_user table
-- 3.   insert into the abh.stage_task_type table
-- 4.   insert into the abh.stage_d_trans table
-- 4a.  insert from abh.stage_d_trans table into abh.d_transaction
-- 5.   insert into the abh.stage_applarea table
-- 6.   insert into the abh.stage_applname table
-- 7.   insert into the abh.stage_d_appl_group table
-- 7a.  insert from abh.stage_d_appl_group table into abh.d_appl_group
-- 8.   insert into the abh.stage_d_tx_sla table
-- 8a.  insert from abh.stage_d_tx_sla table into abh.d_tx_sla
-- 9.   set up temp tables used to populate abh.stage_f_trans_h_cp
-- 9a.  insert into the abh.stage_f_trans_h_cp table
-- 10.  set up temp tables used to populate abh.stage_f_trans_h_cp for tx_sla data
-- 10a. insert into the abh.stage_f_trans_h_cp table
-- 11.  set up temp tables used to populate abh.stage_f_trans_h_cp for application data
-- 11a. insert into the abh.stage_f_trans_h_cp table for application data


--Clean up
--#EXECUTE_AT_TARGET
-- log the pruning of the hourly fact table abh.f_trans_hour
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_trans_hour',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_trans_hour'),
  (select count(*) from abh.f_trans_hour f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_hour)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_trans_hour')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_trans_hour f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_hour)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_trans_hour')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the daily fact table abh.f_trans_day
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_trans_day',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_trans_day'),
  (select count(*) from abh.f_trans_day f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_trans_day')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_trans_day f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_trans_day')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the weekly fact table abh.f_trans_week
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_trans_week',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_trans_week'),
  (select count(*) from abh.f_trans_week f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_trans_week')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_trans_week f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_trans_week')
;

--#EXECUTE_AT_TARGET
-- log the pruning of the monthly fact table abh.f_trans_month
insert into abh.prune_mart_log  values ( 
  current timestamp - current timezone,  
  'abh.f_trans_month',
  (select PMartC_Duration from abh.prune_mart_control 
     where Table_Name='abh.f_trans_month'),
  (select count(*) from abh.f_trans_month f  
     where (DATE(CURRENT timestamp - CURRENT timezone) - DATE(f.meas_date)) > 
        (SELECT PMartC_Duration FROM abh.Prune_Mart_Control p
        WHERE p.Table_Name= 'abh.f_trans_month')
  )      
)
;  

--#EXECUTE_AT_TARGET
delete from abh.f_trans_month f
where
   (date(CURRENT timestamp - CURRENT timezone) - date(f.meas_date)) >
    (select PMartC_Duration from abh.prune_mart_control p
     where p.table_name = 'abh.f_trans_month')
;

-- 1.    insert into the metric dimension table
-- NOTE: This process will only happen once. We initially insert the data into the
--       staging table in the cdw/dml script.
--       set the high value for the data to be extracted

-- 1a.  update extract control for the transaction metric dimension table
-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

update TWG.Extract_Control set 
 extctl_to_intseq = (select max(metric_id) from ABH.STAGE_D_METRIC),
 extctl_to_dttm = (select max(insert_dttm) from ABH.STAGE_D_METRIC)
where 
 extctl_source='ABH.STAGE_D_METRIC' and
 extctl_target='ABH.D_TRANS_METRIC' and
 (select count(*) from ABH.STAGE_D_METRIC) > 0
;

--#INSERT_INTO_TARGET
insert into abh.d_trans_metric
select distinct
  d.metric_id as metric_id,
  d.Met_Name as Met_Name,
  d.Met_Desc as Met_Desc,
  d.Met_units as Met_units,
  d.Met_Category as Met_Category,
  d.min_exists as MIN_Exists,
  d.max_exists as MAX_Exists,
  d.avg_exists as AVG_Exists,
  d.total_exists as TOTAL_Exists,
  d.Msrc_Nm as MSrc_Nm
from
  abh.stage_d_metric d,
  twg.extract_control ec
where
  d.met_name in ('CPU time','Response time','Database time','kB transferred','GUI time',
    'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
    'Dialog count','Within SLA') and
  ec.extctl_source = 'ABH.STAGE_D_METRIC' and
  ec.extctl_target = 'ABH.D_TRANS_METRIC' and
  d.metric_id > ec.extctl_from_intseq and
  d.metric_id <= ec.extctl_to_intseq and
  d.met_category not like 'mySAP%'
;

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'ABH.STAGE_D_METRIC' and
 ec.extctl_target = 'ABH.D_TRANS_METRIC'
; 

-- 1b.  update extract control for the appl group metric dimension table
update TWG.Extract_Control set 
 extctl_to_intseq = (select max(metric_id) from ABH.STAGE_D_METRIC),
 extctl_to_dttm = (select max(insert_dttm) from ABH.STAGE_D_METRIC)
where 
 extctl_source='ABH.STAGE_D_METRIC' and
 extctl_target='ABH.D_APPLGROUP_METRIC' and
 (select count(*) from ABH.STAGE_D_METRIC) > 0
;

--#INSERT_INTO_TARGET
insert into abh.d_applgroup_metric
select distinct
  d.metric_id as metric_id,
  d.Met_Name as Met_Name,
  d.Met_Desc as Met_Desc,
  d.Met_units as Met_units,
  d.Met_Category as Met_Category,
  d.min_exists as MIN_Exists,
  d.max_exists as MAX_Exists,
  d.avg_exists as AVG_Exists,
  d.total_exists as TOTAL_Exists,
  d.Msrc_Nm as MSrc_Nm
from
  abh.stage_d_metric d,
  twg.extract_control ec
where
  d.met_name in ('CPU time','Response time','Database time','kB transferred','GUI time',
    'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
    'Dialog count') and
  ec.extctl_source = 'ABH.STAGE_D_METRIC' and
  ec.extctl_target = 'ABH.D_APPLGROUP_METRIC' and
  d.metric_id > ec.extctl_from_intseq and
  d.metric_id <= ec.extctl_to_intseq and
  d.met_category not like 'mySAP%'
;

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'ABH.STAGE_D_METRIC' and
 ec.extctl_target = 'ABH.D_APPLGROUP_METRIC'
; 

-- 1c.  update extract control for the tx sla metric dimension table
update TWG.Extract_Control set 
 extctl_to_intseq = (select max(metric_id) from ABH.STAGE_D_METRIC),
 extctl_to_dttm = (select max(insert_dttm) from ABH.STAGE_D_METRIC)
where 
 extctl_source='ABH.STAGE_D_METRIC' and
 extctl_target='ABH.D_TX_SLA_METRIC' and
 (select count(*) from ABH.STAGE_D_METRIC) > 0
;

--#INSERT_INTO_TARGET
insert into abh.d_tx_sla_metric
select distinct
  d.metric_id as metric_id,
  d.Met_Name as Met_Name,
  d.Met_Desc as Met_Desc,
  d.Met_units as Met_units,
  d.Met_Category as Met_Category,
  d.min_exists as MIN_Exists,
  d.max_exists as MAX_Exists,
  d.avg_exists as AVG_Exists,
  d.total_exists as TOTAL_Exists,
  d.Msrc_Nm as MSrc_Nm
from
  abh.stage_d_metric d,
  twg.extract_control ec
where
  d.met_name in ('CPU time','Response time','Database time','kB transferred','GUI time',
    'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
    'Dialog count','Within SLA') and
  ec.extctl_source = 'ABH.STAGE_D_METRIC' and
  ec.extctl_target = 'ABH.D_TX_SLA_METRIC' and
  d.metric_id > ec.extctl_from_intseq and
  d.metric_id <= ec.extctl_to_intseq and
  d.met_category not like 'mySAP%'
;

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'ABH.STAGE_D_METRIC' and
 ec.extctl_target = 'ABH.D_TX_SLA_METRIC'
; 

-- 2.  insert into the abh.stage_user table
declare global temporary table stage_user (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 user_name            varchar(30),
 cust_acct_cd         varchar(10),
 centr_cd             varchar(6),
 meas_date            timestamp not null,
 end_dttm             timestamp not null,
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_user
select 
 c3.comp_corr_val as system_label,
 c3.comp_nm as ip_host,
 rtrim(c1.comp_corr_val) as rfc_dest_str,
 rtrim(c1.comp_nm) as user_name,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as meas_date,
 '9999-01-01-00.00.00.000000' as end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 twg.cur_comp c2,
 twg.cur_comp c3,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 c1.comptyp_cd='ABH_USER' and 
 c1.comp_corr_id = c2.comp_id and
 c2.comp_corr_id = c3.comp_id and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 3.  insert into the abh.stage_task_type table
declare global temporary table stage_task_type (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 user_name            varchar(30),
 task_type            varchar(1),
 cust_acct_cd         varchar(10),
 centr_cd             varchar(6),
 meas_date            timestamp not null,
 end_dttm             timestamp not null,
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_task_type
select 
 abh.system_label as system_label,
 abh.ip_host as ip_host,
 abh.rfc_dest_str as rfc_dest_str,
 rtrim(c1.comp_corr_val) as user_name,
 rtrim(c1.comp_nm) as task_type,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as meas_date,
 '9999-01-01-00.00.00.000000' as end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.stage_user abh
where 
 c1.comptyp_cd='ABH_TASKTYPE' and 
 c1.comp_corr_id = abh.comp_id and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 4.  insert into the abh.stage_d_trans table
--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_d_trans;

--#EXECUTE_AT_TARGET
create table abh.stage_d_trans like abh.d_transaction;

--#INSERT_INTO_TARGET
insert into abh.stage_d_trans
select 
 abh.system_label as system_label,
 abh.ip_host as ip_host,
 abh.rfc_dest_str as rfc_dest_str,
 abh.user_name as user_name,
 rtrim(c1.comp_corr_val) as task_type,
 rtrim(c1.comp_nm) as t_code,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as xq_strt_dttm,
 '9999-01-01-00.00.00.000000' as xq_end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.stage_task_type abh
where 
 c1.comptyp_cd='ABH_TRANSACTION' and 
 c1.comp_corr_id = abh.comp_id and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 4a. insert from abh.stage_d_trans table into abh.d_transaction
--#EXECUTE_AT_TARGET
insert into abh.d_transaction 
select 
 stage.system_label as system_label,
 stage.ip_host as ip_host,
 stage.rfc_dest_str as rfc_dest_str,
 stage.user_name as user_name,
 stage.task_type as task_type,
 stage.t_code as t_code,
 stage.cust_acct_cd as cust_acct_cd,
 stage.centr_cd as centr_cd,
 stage.xq_strt_dttm as xq_strt_dttm,
 stage.xq_end_dttm as xq_end_dttm,
 stage.comp_id as comp_id
from 
 abh.stage_d_trans stage
where
 not exists (
  select 1 
  from
   abh.d_transaction c
  where
   c.comp_id = stage.comp_id)
;

-- 5.  insert into the abh.stage_applarea table
declare global temporary table stage_applarea (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 group_area           varchar(2),
 cust_acct_cd         varchar(10),
 centr_cd             varchar(6),
 meas_date            timestamp not null,
 end_dttm             timestamp not null,
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_applarea
select 
 c3.comp_corr_val as system_label,
 c2.comp_corr_val as ip_host,
 rtrim(c1.comp_corr_val) as rfc_dest_str,
 rtrim(c1.comp_nm) as group_area,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as meas_date,
 '9999-01-01-00.00.00.000000' as end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 twg.cur_comp c2,
 twg.cur_comp c3,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust
where 
 c1.comptyp_cd='ABH_APPLAREA' and 
 c1.comp_corr_id = c2.comp_id and
 c2.comp_corr_id = c3.comp_id and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 6.  insert into the abh.stage_applname table
declare global temporary table stage_applname (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 group_area           varchar(2),
 group_name           varchar(80),
 cust_acct_cd         varchar(10),
 centr_cd             varchar(6),
 meas_date            timestamp not null,
 end_dttm             timestamp not null,
 comp_id              integer,
 comp_corr_id         integer,
 comp_corr_val        varchar(254)
   )
  with replace on commit preserve rows not logged;

insert into session.stage_applname
select 
 abh.system_label as system_label,
 abh.ip_host as ip_host,
 abh.rfc_dest_str as rfc_dest_str,
 rtrim(c1.comp_corr_val) as group_area,
 rtrim(c1.comp_nm) as group_name,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as meas_date,
 '9999-01-01-00.00.00.000000' as end_dttm,
 c1.comp_id as comp_id,
 c1.comp_corr_id as comp_corr_id,
 c1.comp_corr_val as comp_corr_val
from 
 twg.cur_comp c1,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.stage_applarea abh
where 
 c1.comptyp_cd='ABH_APPLNAME' and 
 c1.comp_corr_id = abh.comp_id and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 7.  insert into the abh.stage_d_appl_group table
--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_d_appl_group;

--#EXECUTE_AT_TARGET
create table abh.stage_d_appl_group like abh.d_appl_group;

--#INSERT_INTO_TARGET
insert into abh.stage_d_appl_group
select 
 abh.system_label as system_label,
 abh.ip_host as ip_host,
 abh.rfc_dest_str as rfc_dest_str,
 abh.group_area as group_area,
 abh.group_name as group_name,
 rtrim(c1.comp_nm) as t_code,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as xq_strt_dttm,
 '9999-01-01-00.00.00.000000' as xq_end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 twg.cur_comp c2,
 twg.cur_comp c3,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 session.stage_applname abh
where 
 c1.comptyp_cd='ABH_TRANSACTION' and 
 c1.comp_corr_val = abh.group_name and
 c1.comp_corr_id = abh.comp_id and
 c2.comp_id = abh.comp_corr_id and
 c2.comp_nm = abh.group_area and
 c3.comp_id = c2.comp_corr_id and
 c3.comp_nm = abh.rfc_dest_str and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

-- 7a. insert from abh.stage_d_appl_group table into abh.d_appl_group
--#EXECUTE_AT_TARGET
insert into abh.d_appl_group
select 
 stage.system_label as system_label,
 stage.ip_host as ip_host,
 stage.rfc_dest_str as rfc_dest_str,
 stage.group_area as group_area,
 stage.group_name as group_name,
 stage.t_code as t_code,
 stage.cust_acct_cd as cust_acct_cd,
 stage.centr_cd as centr_cd,
 stage.xq_strt_dttm as xq_strt_dttm,
 stage.xq_end_dttm as xq_end_dttm,
 stage.comp_id as comp_id
from 
 abh.stage_d_appl_group stage
where
 not exists (
  select 1 
  from
   abh.d_appl_group c
  where
   c.comp_id = stage.comp_id)
;

-- 8.  insert into the abh.stage_d_tx_sla table
--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_d_tx_sla;

--#EXECUTE_AT_TARGET
create table abh.stage_d_tx_sla like abh.d_tx_sla;

declare global temporary table stage_system_label (
 system_label         varchar(34),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_system_label
select
 c1.Comp_Nm as system_label,
 c1.Comp_Id as comp_id
from
 twg.cur_comp c1
where
 c1.CompTyp_Cd = 'ABH_SYSTEM'
;

declare global temporary table stage_SysGroups (
 system_label         varchar(34),
 group_name           varchar(80),
 comp_id              integer,
 comp_corr_id         integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_SysGroups
select
 comp.Comp_Corr_val as system_label,
 comp.Comp_Nm as group_name,
 comp.Comp_Id as comp_id,
 comp.Comp_Corr_Id as comp_corr_id
from
 twg.cur_comp comp,
 session.stage_system_label sys
where
 comp.CompTyp_Cd = 'ABH_APPLNAME' and 
 comp.Comp_Corr_Id = sys.comp_id
;

declare global temporary table stage_SysGroupTcde (
 system_label         varchar(34),
 group_name           varchar(80),
 t_code               varchar(20),
 comp_id              integer,
 comp_corr_id         integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_SysGroupTcde
select
 sys.system_label as system_label,
 comp.Comp_Corr_val as group_name,
 comp.Comp_Nm as t_code,
 comp.Comp_Id as comp_id,
 comp.Comp_Corr_Id as comp_corr_id
from
 twg.cur_comp comp,
 session.stage_SysGroups sys
where
 comp.CompTyp_Cd = 'ABH_TRANSACTION' and 
 comp.Comp_Corr_Id = sys.comp_id
;

-- update the to column (records to extract) with the max sequence 
-- number in the source data
update TWG.extract_control
  set extctl_to_intseq = (select max(comp_id) from twg.cur_comp)
where
  extctl_source='TWG.COMP' and
  extctl_target='ABH.STAGE_TX_SLA'
;

--#INSERT_INTO_TARGET
insert into abh.stage_d_tx_sla
select 
 abh.system_label as system_label,
 abh.group_name as group_name,
 rtrim(c1.comp_nm) as t_code,
 cust_l.cust_acct_cd as cust_acct_cd,
 centr_l.centr_cd as centr_cd,
 current timestamp - current timezone as xq_strt_dttm,
 '9999-01-01-00.00.00.000000' as xq_end_dttm,
 c1.comp_id as comp_id
from 
 twg.cur_comp c1,
 twg.cur_comp c2,
 abh.centr_lookup centr_l,
 abh.cust_lookup cust_l,
 twg.cust cust,
 twg.extract_control ec,
 session.stage_SysGroupTcde abh
where 
 ec.extctl_source='TWG.COMP' and
 ec.extctl_target='ABH.STAGE_TX_SLA' and
 c1.comp_id > ec.extctl_from_intseq and 
 c1.comp_id <= ec.extctl_to_intseq and
 c1.comptyp_cd='ABH_TRANSACTION' and 
 c1.comp_corr_val = abh.group_name and
 c2.comp_id = c1.comp_corr_id and
 c2.comp_corr_val = abh.system_label and
 ((cust_l.value = c1.comp_nm AND cust_l.Cust_Acct_Cd = cust.Cust_Acct_Cd) 
      OR cust_l.value = '@')
 and (centr_l.value = c1.comp_nm OR centr_l.value = '@')
;

insert into TWG.extract_log
select 
 extctl_source, extctl_target,
 current timestamp - current timezone,
 extctl_from_rawseq, extctl_to_rawseq,
 extctl_from_intseq, extctl_to_intseq,
 extctl_from_dttm, extctl_to_dttm
from
 TWG.extract_control ec
where
  extctl_source='TWG.COMP' and
  extctl_target='ABH.STAGE_TX_SLA'
;

-- 8a. insert from abh.stage_d_tx_sla table into abh.d_tx_sla
--#EXECUTE_AT_TARGET
insert into abh.d_tx_sla
select distinct
 stage.system_label as system_label,
 stage.group_name as group_name,
 stage.t_code as t_code,
 stage.cust_acct_cd as cust_acct_cd,
 stage.centr_cd as centr_cd,
 stage.xq_strt_dttm as xq_strt_dttm,
 stage.xq_end_dttm as xq_end_dttm,
 stage.comp_id as comp_id
from 
 abh.stage_d_tx_sla stage
where
 not exists (
  select 1 
  from
   abh.d_tx_sla c
  where
   c.comp_id = stage.comp_id)
;

-- 9.  set up temp tables used to populate abh.stage_f_trans_h_cp
declare global temporary table stage_ms_ip_host (
 system_label         varchar(34),
 ip_host              varchar(55),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_ms_ip_host
select 
 comp.comp_corr_val as system_label,
 comp.comp_nm as ip_host,
 comp.comp_id as comp_id
from
 twg.cur_comp comp,
 twg.cur_comp comp2
where
 comp.comptyp_cd = 'IP_HOST' and
 comp.comp_corr_id = comp2.comp_id and
 comp2.comptyp_cd = 'ABH_SYSTEM'
;

declare global temporary table stage_ms_tcode (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 user_name            varchar(30),
 task_type            varchar(1),
 t_code               varchar(20),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_ms_tcode
select 
 stage.system_label as system_label,
 comp4.comp_corr_val as ip_host,
 comp4.comp_nm as rfc_dest_str,
 comp3.comp_nm as user_name,
 comp2.comp_nm as task_type,
 comp.comp_nm as t_code,
 comp.comp_id as comp_id
from
 twg.cur_comp comp,
 twg.cur_comp comp2,
 twg.cur_comp comp3,
 twg.cur_comp comp4,
 twg.cur_comp comp5,
 session.stage_ms_ip_host stage
where
 comp.comptyp_cd = 'ABH_TRANSACTION' and
 comp.comp_corr_id = comp2.comp_id and
 comp2.comptyp_cd = 'ABH_TASKTYPE' and
 comp2.comp_corr_id = comp3.comp_id and
 comp3.comptyp_cd = 'ABH_USER' and
 comp3.comp_corr_id = comp4.comp_id and
 comp4.comptyp_cd = 'ABH_SERVER' and
 comp4.comp_corr_val = stage.ip_host and
 comp4.comp_corr_id = comp5.comp_id and
 comp5.comptyp_cd = 'IP_HOST' and
 comp5.comp_corr_val = stage.system_label
;

-- 9a. insert into the abh.stage_f_trans_h_cp table
-- set the high value for the data to be extracted
update TWG.extract_control set 
 extctl_to_intseq = ( select max(msmt_id) from twg.msmt)
where 
 extctl_source='TWG.MSMT' and
 extctl_target='ABH.F_TRANSACTION_HOUR' and
 (select count(*) from twg.msmt) > 0
;

declare global temporary table temp_f_trans_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

insert into session.temp_f_trans_seq
select 
 a.extctl_from_intseq as SeqID_from,
 a.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a
where 
  a.extctl_source='TWG.MSMT' and
  a.extctl_target='ABH.F_TRANSACTION_HOUR'
;

--#IGNORE_ERROR
--#EXECUTE_AT_TARGET
drop table abh.stage_f_trans_hour;

--#EXECUTE_AT_TARGET
create table abh.stage_f_trans_hour like abh.f_trans_hour;

--#INSERT_INTO_TARGET
insert into abh.stage_f_trans_hour
select
  ms.comp_id as transaction_id,
  ms.msmttyp_id as metric_id,
  timestamp(ms.msmt_strt_dt, ms.msmt_strt_tm) as meas_hour,   
  ms.msmt_smpl_cnt as sample_count,   
  ms.msmt_min_val as min_value,
  ms.msmt_max_val as max_value,
  ms.msmt_avg_val as avg_value,
  ms.msmt_tot_val as total_value
from
  twg.msmt ms, 
  twg.msmttyp mt, 
  session.stage_ms_tcode stage,
  session.temp_f_trans_seq t_seq
where
  stage.comp_id = ms.comp_id and
  ms.msmt_id > t_seq.SeqID_from and 
  ms.msmt_id <= t_seq.SeqID_to and
  (ms.msmttyp_id = mt.msmttyp_id and mt.msrc_cd = 'ABH' and 
   mt.msmttyp_nm in ('CPU time','Response time','Database time','kB transferred','GUI time',
     'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
     'Within SLA','Dialog count') ) and
  ms.tmsum_cd = 'H'
;

--Note: the insert into the ABH.F_TRANS_HOUR will take place in step "10a" after we
--      have added the tx sla information to this table. The update of the extract control
--      log is also located there


-- 10.  set up temp tables used to populate abh.stage_f_trans_h_cp
--      for transactional SLA

declare global temporary table stage_group_tcodes (
 system_label         varchar(34),
 group_name           varchar(80),
 t_code               varchar(20),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_group_tcodes
select
 comp3.Comp_Nm as system_label,
 comp2.Comp_Nm as group_name,
 comp1.Comp_Nm as t_code,
 comp1.Comp_Id as comp_id
from
 twg.cur_comp comp1,
 twg.cur_comp comp2,
 twg.cur_comp comp3
where
 comp1.comptyp_cd = 'ABH_TRANSACTION' and
 comp1.comp_corr_id = comp2.comp_id and
 comp2.comptyp_cd = 'ABH_APPLNAME' and
 comp2.comp_corr_id = comp3.comp_id and
 comp3.comptyp_cd = 'ABH_SYSTEM' 
;

-- 10a. insert into the abh.stage_f_trans_h_cp table
-- set the high value for the data to be extracted
update TWG.extract_control set 
 extctl_to_intseq = ( select max(msmt_id) from twg.msmt)
where 
 extctl_source='TWG.MSMT' and
 extctl_target='ABH.F_TX_SLA_HOUR' and
 (select count(*) from twg.msmt) > 0
;

declare global temporary table temp_f_tx_sla_seq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

insert into session.temp_f_tx_sla_seq
select 
 a.extctl_from_intseq as SeqID_from,
 a.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a
where 
  a.extctl_source='TWG.MSMT' and
  a.extctl_target='ABH.F_TX_SLA_HOUR'
;

--#INSERT_INTO_TARGET
insert into abh.stage_f_trans_hour
select
  ms.comp_id as transaction_id,
  ms.msmttyp_id as metric_id,
  timestamp(ms.msmt_strt_dt, ms.msmt_strt_tm) as meas_hour,   
  ms.msmt_smpl_cnt as sample_count,   
  ms.msmt_min_val as min_value,
  ms.msmt_max_val as max_value,
  ms.msmt_avg_val as avg_value,
  ms.msmt_tot_val as total_value
from
  twg.msmt ms, 
  twg.msmttyp mt, 
  session.stage_group_tcodes stage,
  session.temp_f_tx_sla_seq t_seq
where
  stage.comp_id = ms.comp_id and
  ms.msmt_id > t_seq.SeqID_from and 
  ms.msmt_id <= t_seq.SeqID_to and
  (ms.msmttyp_id = mt.msmttyp_id and mt.msrc_cd = 'ABH' and 
   mt.msmttyp_nm in ('CPU time','Response time','Database time','kB transferred','GUI time',
    'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
    'Dialog count','Within SLA') ) and
  ms.tmsum_cd = 'H'
;

-- 11.  set up temp tables used to populate abh.stage_f_trans_h_cp
--      for application

declare global temporary table stage_appl_group (
 system_label         varchar(34),
 ip_host              varchar(55),
 rfc_dest_str         varchar(20),
 group_area           varchar(2),
 group_name           varchar(80),
 t_code               varchar(20),
 comp_id              integer
   )
  with replace on commit preserve rows not logged;

insert into session.stage_appl_group
select
 comp6.Comp_Nm as system_label,
 comp5.Comp_Nm as ip_host,
 comp4.Comp_Nm as rfc_dest_str,
 comp3.Comp_Nm as group_area,
 comp2.Comp_Nm as group_name,
 comp1.Comp_Nm as t_code,
 comp1.Comp_Id as comp_id
from
 twg.cur_comp comp1,
 twg.cur_comp comp2,
 twg.cur_comp comp3,
 twg.cur_comp comp4,
 twg.cur_comp comp5,
 twg.cur_comp comp6
where
 comp1.comptyp_cd = 'ABH_TRANSACTION' and
 comp1.comp_corr_id = comp2.comp_id and
 comp2.comptyp_cd = 'ABH_APPLNAME' and
 comp2.comp_corr_id = comp3.comp_id and
 comp3.comptyp_cd = 'ABH_APPLAREA' and
 comp3.comp_Nm != 'ZZ' and
 comp3.comp_corr_id = comp4.comp_id and
 comp4.comptyp_cd = 'ABH_SERVER' and
 comp4.comp_corr_id = comp5.comp_id and
 comp5.comptyp_cd = 'IP_HOST' and
 comp5.comp_corr_id = comp6.comp_id and
 comp6.comptyp_cd = 'ABH_SYSTEM' 
;

-- 11a. insert into the abh.stage_f_trans_h_cp table for application data
-- set the high value for the data to be extracted
update TWG.extract_control set 
 extctl_to_intseq = ( select max(msmt_id) from twg.msmt)
where 
 extctl_source='TWG.MSMT' and
 extctl_target='ABH.F_APPLICATION_HOUR' and
 (select count(*) from twg.msmt) > 0
;

declare global temporary table tmp_f_appl_sq (
        SeqID_from           integer,
        SeqID_to             integer
        )
  with replace on commit preserve rows not logged;

insert into session.tmp_f_appl_sq
select 
 a.extctl_from_intseq as SeqID_from,
 a.extctl_to_intseq as SqdID_to
from TWG.Extract_Control a
where 
  a.extctl_source='TWG.MSMT' and
  a.extctl_target='ABH.F_APPLICATION_HOUR'
;

--#INSERT_INTO_TARGET
insert into abh.stage_f_trans_hour
select
  ms.comp_id as transaction_id,
  ms.msmttyp_id as metric_id,
  timestamp(ms.msmt_strt_dt, ms.msmt_strt_tm) as meas_hour,   
  ms.msmt_smpl_cnt as sample_count,
  ms.msmt_min_val as min_value,
  ms.msmt_max_val as max_value,
  ms.msmt_avg_val as avg_value,
  ms.msmt_tot_val as total_value
from
  twg.msmt ms, 
  twg.msmttyp mt, 
  session.stage_appl_group stage,
  session.tmp_f_appl_sq t_seq
where
  stage.comp_id = ms.comp_id and
  ms.msmt_id > t_seq.SeqID_from and 
  ms.msmt_id <= t_seq.SeqID_to and
  (ms.msmttyp_id = mt.msmttyp_id and mt.msrc_cd = 'ABH' and 
   mt.msmttyp_nm in ('CPU time','Response time','Database time','kB transferred','GUI time',
    'kB changed','Wait time','Memory used','Load time','GUI network time','Other time',
    'Dialog count')) and
  ms.tmsum_cd = 'H'
;

--#EXECUTE_AT_TARGET
update abh.stage_f_trans_hour 
  set total_value = avg_value * sample_count 
where 
  total_value is null;

--#EXECUTE_AT_TARGET
insert into ABH.F_TRANS_HOUR
select * from abh.stage_f_trans_hour
;

-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'TWG.MSMT' and
 ec.extctl_target = 'ABH.F_TX_SLA_HOUR'
; 

-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'TWG.MSMT' and
 ec.extctl_target = 'ABH.F_APPLICATION_HOUR'
; 

-- Confirm the insertion of data by inserting a row into TWG.EXTRACT_LOG.
-- A trigger gets fired which updates the extract control table's from data with
-- the current to data.

insert into twg.extract_log
select
 ec.extctl_source as extlog_source,
 ec.extctl_target as extlog_target,
 current timestamp - current timezone as extlog_done_dttm,
 ec.extctl_from_rawseq as extlog_from_rawseq,
 ec.extctl_to_rawseq as extlog_to_rawseq,
 ec.extctl_from_intseq as extlog_from_intseq,
 ec.extctl_to_intseq as extlog_to_intseq,
 ec.extctl_from_dttm as extlog_from_dttm,
 ec.extctl_to_dttm as extlog_to_dttm
from 
 TWG.Extract_control ec
where
 ec.extctl_source = 'TWG.MSMT' and
 ec.extctl_target = 'ABH.F_TRANSACTION_HOUR'
; 
