-- This is a sample sql script for adding multi-customer support.
-- It is integrated into the prototype installation to make life easier,
-- but that would not be the typical approach.


-Remove the DEFAULT single customer/center environment.

delete from spp.cust_lookup where cust_acct_cd='DEF';
delete from spp.centr_lookup where centr_cd='DEF';
delete from inv.cust_lookup where cust_acct_cd='DEF';
delete from inv.centr_lookup where centr_cd='DEF';


-- Insert data into lookup tables to setup a multi customer/center environment.

-- To support multiple timezone's and customers we are using the hostname as 
-- a lookup value to determine the center and customer.

-- I'm not using the "where not exists" clause when inserting this data.
-- If it's duplicate, I want to know and pick a unique customer and center code
-- for my application.

--shouldn't need to do this...cust_acct_cd should be a unique value
delete from twg.cust where cust_acct_cd = 'ABB' or cust_acct_cd = 'TIV';

insert into twg.cust
  (cust_id, cust_parent_id, geoarea_cd, tmzon_id, cust_acct_cd, cust_nm)
  values (0, NULL, 'EUR', 25, 'ABB', 'ABB Bank'
);
insert into twg.cust
  (cust_id, cust_parent_id, geoarea_cd, tmzon_id, cust_acct_cd, cust_nm)
  values (1, NULL, 'US', 8, 'TIV', 'Tivoli'
);

--the inserts into centr will fail if TIV, ABB centr codes are already defined.
insert into twg.centr
  (centr_cd, centr_parent_cd, geoarea_cd, tmzon_id, centr_nm)
  values (
  'TIV', NULL, 'US', 8, 'Tivoli Austin data center'
);
insert into twg.centr
  (centr_cd, centr_parent_cd, geoarea_cd, tmzon_id, centr_nm)
  values (
  'ABB', NULL, 'EUR', 25, 'Belgium ABB Bank data center'
);

-- For the prototype we are hardcoding the relation between 
-- customers/centers and hostnames.
-- Usually this would be determined programatically and populated
-- something like the smple below:

--insert into inv.cust_lookup
--select 0, t.hostname  <--setting all custIDs to the same value, 0 
--from temp_hostname t  <--temp_hostname pop'd from DNS or some other app
--where
-- not exists (
--  select 1 from
--  inv_cust_lookup c
--  where
--      c.value=t.hostname
--  )
--;

delete from inv.cust_lookup;

--hosts correspond to the sample data that is loaded into dm_metrics
insert into inv.cust_lookup
values ('TIV', 'colbert');
insert into inv.cust_lookup
values ('TIV', 'hpuxs0.tivoli.com');
insert into inv.cust_lookup
values ('TIV', 'jaba95f1.dev.tivoli.com');
insert into inv.cust_lookup
values ('ABB', 'ja0ce102.leuven.add.abb.be');
insert into inv.cust_lookup
values ('ABB', 'ja0ce201.leuven.add.abb.be');
insert into inv.cust_lookup
values ('ABB', 'ja0np204.centrale.abb.be');
insert into inv.cust_lookup
values ('ABB', 'ja1fe204.brussel.add.abb.be');
insert into inv.cust_lookup
values ('ABB', 'ja5cp102.hasselt.abb.be');

delete from inv.centr_lookup;

insert into inv.centr_lookup
values ('TIV', 'colbert');
insert into inv.centr_lookup
values ('TIV', 'hpuxs0.tivoli.com');
insert into inv.centr_lookup
values ('TIV', 'jaba95f1.dev.tivoli.com');
insert into inv.centr_lookup
values ('ABB', 'ja0ce102.leuven.add.abb.be');
insert into inv.centr_lookup
values ('ABB', 'ja0ce201.leuven.add.abb.be');
insert into inv.centr_lookup
values ('ABB', 'ja0np204.centrale.abb.be');
insert into inv.centr_lookup
values ('ABB', 'ja1fe204.brussel.add.abb.be');
insert into inv.centr_lookup
values ('ABB', 'ja5cp102.hasselt.abb.be');

delete from spp.cust_lookup;

insert into spp.cust_lookup
values ('TIV', 'colbert');
insert into spp.cust_lookup
values ('TIV', 'hpuxs0.tivoli.com');
insert into spp.cust_lookup
values ('TIV', 'jaba95f1.dev.tivoli.com');
insert into spp.cust_lookup
values ('ABB', 'ja0ce102.leuven.add.abb.be');
insert into spp.cust_lookup
values ('ABB', 'ja0ce201.leuven.add.abb.be');
insert into spp.cust_lookup
values ('ABB', 'ja0np204.centrale.abb.be');
insert into spp.cust_lookup
values ('ABB', 'ja1fe204.brussel.add.abb.be');
insert into spp.cust_lookup
values ('ABB', 'ja5cp102.hasselt.abb.be');

delete from spp.centr_lookup;

insert into spp.centr_lookup
values ('TIV', 'colbert');
insert into spp.centr_lookup
values ('TIV', 'hpuxs0.tivoli.com');
insert into spp.centr_lookup
values ('TIV', 'jaba95f1.dev.tivoli.com');
insert into spp.centr_lookup
values ('ABB', 'ja0ce102.leuven.add.abb.be');
insert into spp.centr_lookup
values ('ABB', 'ja0ce201.leuven.add.abb.be');
insert into spp.centr_lookup
values ('ABB', 'ja0np204.centrale.abb.be');
insert into spp.centr_lookup
values ('ABB', 'ja1fe204.brussel.add.abb.be');
insert into spp.centr_lookup
values ('ABB', 'ja5cp102.hasselt.abb.be');

-- In the mart db you need to create views for each customer. 
-- Then we'll create a separate star schema for
-- each customer to partition the data that users can see.
-- The metric table can be shared across customers, so no views are created.

-- Note: the creation of these views will fail for the prototype
-- because they were created during the installation. The tag file already
-- has warehouse schemas defined with these views for customers ABB and TIV.

create view spp.d_V_ABB_host_state
  as select * from spp.d_host_state
  where cust_acct_cd = 'ABB';
create view spp.d_V_TIV_host_state 
  as select * from spp.d_host_state
  where cust_acct_cd = 'TIV';

-- For RPI the hour tables must end in "_hour"
create view spp.f_V_ABB_hour
  as select * from spp.f_hour 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'ABB');

create view spp.f_V_TIV_hour 
  as select * from spp.f_hour 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'TIV');


create view spp.f_V_day_ABB 
  as select * from spp.f_day 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'ABB');

create view spp.f_V_day_TIV 
  as select * from spp.f_day 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'TIV');


create view spp.f_V_week_ABB 
  as select * from spp.f_week 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'ABB');

create view spp.f_V_week_TIV 
  as select * from spp.f_week 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'TIV');


create view spp.f_V_month_ABB 
  as select * from spp.f_month 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'ABB');

create view spp.f_V_month_TIV 
  as select * from spp.f_month 
  where 
  host_id IN (select host_id from spp.d_host_state where cust_acct_cd = 'TIV');

--Last step is to define new warehouse star schemas using the views in the DWC.
--For the prototype, we've already defined them and they are imported at
--install time.